# -*- coding: utf-8 -*-
"""
Created on Sat Jan 09 19:50:57 2016

@author: marcus

Settings file for the HiSpeed1250 eye tracker
Here we list parameters that can be sent remotely to iView X to override default parameters
OBS! PsychoPy geometric setup should match that entered in iView

"""


MY_MONITOR                  = 'default' # needs to exists in PsychoPy monitor center

#%% CONNECTION AND TRACKER PARAMS

# This should mirror settings in iView. You also need to set the IP addresses on both computers (if two computer setup)
port_listen=4444
port_send=5555
ip_send = '192.168.0.2'
ip_listen = '192.168.0.1'

sampling_freq = 500 # Sampling rate of eye tracker

# The following commands do not seem to work. Change manually in iview instead (setup -> tracking).
average_data = False
filtering = False # You need to turn off the 'bilateral' filter manually

#%% Files and filepaths
temp_folder_path = r"C:\\ProgramData\SMI\iViewX\temp"
delete_temp_idf_files = False

eye_image_size = (160, 640)# Size of eye images in HiSpeed 1250

#%% CALIBRATION PARAMETERS 
autoaccept = 0  # autoaccept (2), semi autoaccept (1, accept first point) of accept with space bar (0)
n_cal_points = 9  # number of calibration points [5, 9, 13] supported
cal_speed = 1   # pacing of calibration / validation targets 0: slow, 1:fast
select_best_calibration = True # option to run a few calibration and select the best
reset_calibration_points = False # Resets calibration points to default locations
record_data_during_calibration = True
animate_calibration = True # Show static points or animated targets
screen = 1

# Parameters to shift and scale the calibration grid
shift_cal_grid_x = 0
shift_cal_grid_y = 0
scale_cal_grid = 1













# -----------------------------------------------------------------------
#
# (c) Copyright 1997-2016, SensoMotoric Instruments GmbH
# 
# Permission  is  hereby granted,  free  of  charge,  to any  person  or
# organization  obtaining  a  copy  of  the  software  and  accompanying
# documentation  covered  by  this  license  (the  "Software")  to  use,
# reproduce,  display, distribute, execute,  and transmit  the Software,
# and  to  prepare derivative  works  of  the  Software, and  to  permit
# third-parties to whom the Software  is furnished to do so, all subject
# to the following:
# 
# The  copyright notices  in  the Software  and  this entire  statement,
# including the above license  grant, this restriction and the following
# disclaimer, must be  included in all copies of  the Software, in whole
# or  in part, and  all derivative  works of  the Software,  unless such
# copies   or   derivative   works   are   solely   in   the   form   of
# machine-executable  object   code  generated  by   a  source  language
# processor.
# 
# THE  SOFTWARE IS  PROVIDED  "AS  IS", WITHOUT  WARRANTY  OF ANY  KIND,
# EXPRESS OR  IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE  WARRANTIES OF
# MERCHANTABILITY,   FITNESS  FOR  A   PARTICULAR  PURPOSE,   TITLE  AND
# NON-INFRINGEMENT. IN  NO EVENT SHALL  THE COPYRIGHT HOLDERS  OR ANYONE
# DISTRIBUTING  THE  SOFTWARE  BE   LIABLE  FOR  ANY  DAMAGES  OR  OTHER
# LIABILITY, WHETHER  IN CONTRACT, TORT OR OTHERWISE,  ARISING FROM, OUT
# OF OR IN CONNECTION WITH THE  SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# -----------------------------------------------------------------------

def HandleError(ret):
    if ret == 1:
        return
    elif ret == 2:
        pass
        #msg = "no new data available"
    elif ret == 3:
        msg = "calibration or validation was aborted during progress"
    elif ret == 4:
        msg = "server is running"
    elif ret == 5:
        msg = "calibration is not in progress"
    elif ret == 11:
        msg = "window is open"        
    elif ret == 12:
        msg = "window is closed"        
    elif ret == 100:
        msg = "failed to establish connection"        
    elif ret == 101:
        msg = "no connection established"
    elif ret == 102:
        msg = "system is not calibrated"
    elif ret == 103:
        msg = "system is not validated"
    elif ret == 104:
        msg = "no eye tracking application running"
    elif ret == 105:
        msg = "failed to establish connection"
    elif ret == 111:
        msg = "no connection established"        
    elif ret == 112:
        msg = "parameter out of range"
    elif ret == 113:
        msg = "eye tracking device required for this calibration method is not connected"
    elif ret == 114:
        msg = "calibration timeout occurred"
    elif ret == 115:
        msg = "eye tracking is not stable"
    elif ret == 116:
        msg = "insufficient buffer size"
    elif ret == 121:
        msg = "cannot create socket"
    elif ret == 122:
        msg = "cannot connect with socket"        
    elif ret == 123:
        msg = "the defined port is blocked"
    elif ret == 124:
        msg = "failed to delete sockets"
    elif ret == 131:
        msg = "iView X (eyetracking-server) application was not able to response to current request"
    elif ret == 132:
        msg = "invalid version of iView X (eyetracking-server)"
    elif ret == 133:
        msg = "wrong version of iView X (eyetracking-server) application"
    elif ret == 171:
        msg = "failed to access log file"
    elif ret == 181:
        msg = "socket connection failed"
    elif ret == 201:
        msg = "Could not establish connection. Check if Eye Tracker is installed and running."
    elif ret == 191:
        msg = "recording buffer is empty"
    elif ret == 192:
        msg = "recording is activated"
    elif ret == 193:
        msg = "data buffer is full"
    elif ret == 194:
        msg = "iView X (eyetracking-server) application is not ready to record buffer"
    elif ret == 195:
        msg = "paused data buffer"
    elif ret == 201:
        msg = "iView X (eyetracking-server) application was not found"
    elif ret == 202:
        msg = "path for file does not exist"
    elif ret == 203:
        msg = "access denied"
    elif ret == 204:
        msg = "access incomplete"
    elif ret == 205:
        msg = "out of memory"
    elif ret == 211:
        msg = "failed to access eye tracking device"
    elif ret == 212:
        msg = "failed to access eye tracking device"
    elif ret == 213:
        msg = "failed to access port connected to eye tracking device"
    elif ret == 220:
        msg = "failed to open port"
    elif ret == 221:
        msg = "failed to close port"
    elif ret == 222:
        msg = "failed to access AOI data"
    elif ret == 223:
        msg = "AOI not defined"
    elif ret == 250:
        msg = "failed to access requested feature"
    elif ret == 300:
        msg = "function is deprecated"
    elif ret == 400:
        msg = "function or dll not initialized"
    else:
        msg = "Unknown return Code " + str(ret)
        
    if ret != 2:
        print(msg)
    


# -----------------------------------------------------------------------
#
# (c) Copyright 1997-2016, SensoMotoric Instruments GmbH
# 
# Permission  is  hereby granted,  free  of  charge,  to any  person  or
# organization  obtaining  a  copy  of  the  software  and  accompanying
# documentation  covered  by  this  license  (the  "Software")  to  use,
# reproduce,  display, distribute, execute,  and transmit  the Software,
# and  to  prepare derivative  works  of  the  Software, and  to  permit
# third-parties to whom the Software  is furnished to do so, all subject
# to the following:
# 
# The  copyright notices  in  the Software  and  this entire  statement,
# including the above license  grant, this restriction and the following
# disclaimer, must be  included in all copies of  the Software, in whole
# or  in part, and  all derivative  works of  the Software,  unless such
# copies   or   derivative   works   are   solely   in   the   form   of
# machine-executable  object   code  generated  by   a  source  language
# processor.
# 
# THE  SOFTWARE IS  PROVIDED  "AS  IS", WITHOUT  WARRANTY  OF ANY  KIND,
# EXPRESS OR  IMPLIED, INCLUDING  BUT NOT LIMITED  TO THE  WARRANTIES OF
# MERCHANTABILITY,   FITNESS  FOR  A   PARTICULAR  PURPOSE,   TITLE  AND
# NON-INFRINGEMENT. IN  NO EVENT SHALL  THE COPYRIGHT HOLDERS  OR ANYONE
# DISTRIBUTING  THE  SOFTWARE  BE   LIABLE  FOR  ANY  DAMAGES  OR  OTHER
# LIABILITY, WHETHER  IN CONTRACT, TORT OR OTHERWISE,  ARISING FROM, OUT
# OF OR IN CONNECTION WITH THE  SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
# -----------------------------------------------------------------------
# iViewXAPI.py
#
# Demonstrates features of iView API 
# Defines structures 
# Loads iViewXAPI.dll / iViewXAPI64.dll
# This script shows how to set up an experiment with Python 2.7.1 (with ctypes Library) 


from ctypes import *
import platform


#===========================
#		Struct Definition
#===========================

class CSystem(Structure):
	_fields_ = [("samplerate", c_int),
	("iV_MajorVersion", c_int),
	("iV_MinorVersion", c_int),
	("iV_Buildnumber", c_int),
	("API_MajorVersion", c_int),
	("API_MinorVersion", c_int),
	("API_Buildnumber", c_int),
	("iV_ETDevice", c_int)]

class CCalibration(Structure):
	_fields_ = [("method", c_int),
	("visualization", c_int),
	("displayDevice", c_int),
	("speed", c_int),
	("autoAccept", c_int),
	("foregroundBrightness", c_int),
	("backgroundBrightness", c_int),
	("targetShape", c_int),
	("targetSize", c_int),
	("targetFilename", c_char * 256)]

class CEye(Structure):
	_fields_ = [("gazeX", c_double),
	("gazeY", c_double),
	("diam", c_double),
	("eyePositionX", c_double),
	("eyePositionY", c_double),
	("eyePositionZ", c_double)]

class CSample(Structure):
	_fields_ = [("timestamp", c_longlong),
	("leftEye", CEye),
	("rightEye", CEye),
	("planeNumber", c_int)]

class CEvent(Structure):
	_fields_ = [("eventType", c_char),
	("eye", c_char),
	("startTime", c_longlong),
	("endTime", c_longlong),
	("duration", c_longlong),
	("positionX", c_double),
	("positionY", c_double)]

class CAccuracy(Structure):
	_fields_ = [("deviationLX",c_double),
				("deviationLY",c_double),
				("deviationRX",c_double),
				("deviationRY",c_double)]
    
class CImage(Structure):
	_fields_ = [("imageHeight",c_int),
				("imageWidth",c_int),
				("imageSize",c_int),
				("imageBuffer", POINTER(c_char))]
				
    
class CCalibrationPoint(Structure):
    _fields_ = [("number",c_int),
                ("positionX",c_int),
                ("positionY",c_int)]
                
class CEyePosition(Structure):
    _fields_ = [("validity",c_int),
                ("relativePositionX",c_double),  
                ("relativePositionY",c_double),
                ("relativePositionZ",c_double),
                ("positionRatingX",c_double),  
                ("positionRatingY",c_double),
                ("positionRatingZ",c_double)]

class CTrackingStatus(Structure):
    _fields_ = [("timestamp", c_longlong),
                ("leftEye", CEyePosition),
                ("rightEye", CEyePosition),
                ("total", CEyePosition)]

class CREDGeometry(Structure):
    _fields_ = [("redGeometry", c_int),
                ("monitorSize", c_int),
                ("setupName", c_char * 256),
                ("stimX", c_int),
                ("stimY", c_int),
                ("stimHeightOverFloor", c_int),
                ("redHeightOverFloor", c_int),
                ("redStimDist", c_int),
                ("redInclAngle", c_int),
                ("redStimDistHeigh", c_int),
                ("redStimDistDepth", c_int)]

#===========================
#		Loading iViewX.dll 
#===========================

if platform.architecture()[0] == '64bit':
        iViewXAPI = windll.LoadLibrary("iViewXAPI64.dll")
else:
        iViewXAPI = windll.LoadLibrary("iViewXAPI.dll")
        
        

#===========================
#		Initializing Structs
#===========================

systemData = CSystem(0, 0, 0, 0, 0, 0, 0, 0)
calibrationData = CCalibration(5, 1, 0, 0, 1, 20, 239, 1, 15, b"")
leftEye = CEye(0,0,0)
rightEye = CEye(0,0,0)
sampleData = CSample(0,leftEye,rightEye,0)
eventData = CEvent('F'.encode('ascii'), 'L'.encode('ascii'), 0, 0, 0, 0, 0)
#eventData = CEvent(0, 0, 0, 0, 0, 0, 0)
accuracyData = CAccuracy(0,0,0,0)
imageData = CImage(0, 0, 0, None)
currentCalibrationPoint = CCalibrationPoint(0, 0, 0)
leftEyePosition = CEyePosition(0, 0, 0, 0, 0, 0, 0)
rightEyePosition = CEyePosition(0, 0, 0, 0, 0, 0, 0)
binocularEyePosition = CEyePosition(0, 0, 0, 0, 0, 0, 0)
trackingStatus = CTrackingStatus(0, leftEyePosition, rightEyePosition, binocularEyePosition)
redGeometry = CREDGeometry(0, 0, b"", 0, 0, 0, 0, 0, 0, 0, 0)





# -*- coding: utf-8 -*-
"""
Created on Thu Aug 30 10:19:10 2018

@author: Marcus
"""
import numpy as np

blue = tuple(np.array([37, 97, 163]) / 255.0 * 2 - 1)
blue_active = tuple(np.array([11, 122, 244]) / 255.0 * 2 - 1)
green = tuple(np.array([0, 120, 0]) / 255.0 * 2 - 1)
red = tuple(np.array([150, 0, 0]) / 255.0 * 2 - 1)
yellow = tuple(np.array([150, 150, 0]) / 255.0 * 2 - 1)

ET_SAMPLE_RADIUS = 10 # Pixles

# Size of text
TEXT_SIZE = 0.04

# SIze of calibration dots
TARGET_SIZE=20  # in pixles
TARGET_SIZE_INNER=TARGET_SIZE / float(5)  # inner diameter of dot

HEAD_POS_CIRCLE_FIXED_COLOR = blue
HEAD_POS_CIRCLE_FIXED_RADIUS = 0.15

HEAD_POS_CIRCLE_MOVING_COLOR = yellow
HEAD_POS_CIRCLE_MOVING_RADIUS = 0.15

POS_CAL_BUTTON = (0.5, -0.8)
COLOR_CAL_BUTTON =  green
WIDTH_CAL_BUTTON = 0.30
HEIGHT_CAL_BUTTON = 0.08
CAL_BUTTON = 'space'
CAL_BUTTON_TEXT = 'calibrate (spacebar)'

POS_RECAL_BUTTON = (-0.5, -0.8)
COLOR_RECAL_BUTTON =  red
WIDTH_RECAL_BUTTON = 0.30
HEIGHT_RECAL_BUTTON = 0.08
RECAL_BUTTON = 'c'
RECAL_BUTTON_TEXT = 'recalibrate (c)'

# Button for advanced setup
POS_SETUP_BUTTON = (-0.5, -0.8)
COLOR_SETUP_BUTTON = blue
WIDTH_SETUP_BUTTON = 0.30
HEIGHT_SETUP_BUTTON = 0.08
SETUP_BUTTON = 'a'
SETUP_BUTTON_TEXT = 'advanced (a)'

POS_ACCEPT_BUTTON = (0.5, -0.8)
COLOR_ACCEPT_BUTTON = green
WIDTH_ACCEPT_BUTTON = 0.30
HEIGHT_ACCEPT_BUTTON = 0.08
ACCEPT_BUTTON = 'space'
ACCEPT_BUTTON_TEXT = 'accept (spacebar)'

POS_BACK_BUTTON = (-0.5, -0.8)
COLOR_BACK_BUTTON = blue
WIDTH_BACK_BUTTON = 0.30
HEIGHT_BACK_BUTTON = 0.08
BACK_BUTTON = 'b'
BACK_BUTTON_TEXT = 'basic (b)'

POS_GAZE_BUTTON = (0.5, 0.8)
COLOR_GAZE_BUTTON = blue
WIDTH_GAZE_BUTTON = 0.25
HEIGHT_GAZE_BUTTON = 0.08
GAZE_BUTTON = 'g'
GAZE_BUTTON_TEXT = 'show gaze (g)'

SETUP_DOT_OUTER_DIAMETER = 0.05 # Height unit
SETUP_DOT_INNER_DIAMETER = 0.02        

#EYE_IMAGE_SIZE = (640, 512)
EYE_IMAGE_SIZE = (512, 512)




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Load required packages 
from psychopy import core, event, misc
from iViewXAPI import*
from iViewXAPIReturnCodes import* 
import subprocess
import numpy as np
import helpers
import glob
import os
from scipy import misc
from ctypes import *

global buf # Used to keep data in ring buffer

# Numbers used by iView X for identify eye trackers
ET_server_dict = {'iViewX':0, 'iViewXOEM':1, 'iViewNG':2}
ET_device_dict = {'NONE':0, 'RED':1, 'REDm':2, 'RED250mobile':2, 'HiSpeed':3,
                  'MRI':4, 'HED':5, 'Custom':7, 'REDn_Professional':8, 
                  'REDn_Scientific':8}

tracking_mode_dict = {'SMART_BINOCULAR':0, 'MONOCULAR_LEFT':1,
                      'MONOCULAR_RIGHT':2, 'BINOCULAR':3, 
                      'SMART_MONOCULAR':4}   

#%%    
        
class Connect(object):
    """ Basic functionally to communicate with and manage SMI eye trackers
    """
    def __init__(self, in_arg):
        '''
        Constructs an instance of the SMITE interface, with specified settings. 
        If settings is not provided, the name of an eye tracker should 
        be given, e.g., RED-m
        '''
        self.clock = core.Clock()
        self.connect_timeout = 30 # in seconds 
        
        # Define what's supported on the different eye tracker
        self.set_sampling_freq_allowed = True
        self.set_binocular_allowed = True
        self.geom_profile = False
        self.set_tracking_mode_allowed = False
        self.set_cal_positions_allowed = True # TODo
        self.enable_processor_high_performance = True

        # String, i.e., eye tracker name OR settings as argument?
        if isinstance(in_arg, str): # 3ye tracker name
            eye_tracker_name = in_arg
        else:                       # settings
            constants = in_arg
            eye_tracker_name = constants.eye_tracker_name
            
        if eye_tracker_name == 'REDm':
            import REDm as constants
            self.et_server_name = 'iViewXOEM'
            self.set_tracking_mode_allowed = True
            self.geom_profile = True
        elif eye_tracker_name == 'HiSpeed':
            import HiSpeed as constants
            self.enable_processor_high_performance_mode = False
            self.et_server_name = 'iViewX'
            self.set_binocular_allowed = False
            self.enable_processor_high_performance = False
        elif eye_tracker_name == 'RED':
            import RED as constants
            self.et_server_name = 'iViewX'
        elif eye_tracker_name == 'REDn':
            import REDn as constants
            self.et_server_name = 'iViewNG'
            self.set_tracking_mode_allowed = True
            self.geom_profile = True
        elif eye_tracker_name == 'REDn_Professional':
            self.et_server_name = 'iViewNG'
            self.set_tracking_mode_allowed = True
            self.geom_profile = True            
            import REDn_Professional as constants
        elif eye_tracker_name == 'REDn_Scientific':
            self.et_server_name = 'iViewNG'
            import REDn_Scientific as constants  
            self.set_tracking_mode_allowed = True
            self.geom_profile = True            
        elif eye_tracker_name == 'RED250mobile':
            self.et_server_name = 'iViewNG'
            import RED250mobile as constants  
            self.set_tracking_mode_allowed = True
            self.geom_profile = True
        else:
            print('Eye tracker not defined')
            core.quit()   
             
        self.constants = constants
        self.eye_tracker_name = eye_tracker_name
        
        print(self.et_server_name)
        
    #%%
    def init(self):
        ''' Connects to the eye tracker and initializes it according to the 
        specified settings
        '''        
        
        # Connect to eye tracker
        self.connect(self.constants.ip_listen, self.constants.port_listen,
                     self.constants.ip_send, self.constants.port_send)
                
        # Stop recording and clear buffer
        self.stop_recording()
        self.clear_recording_buffer()
        
        # Reset calibration points (also resets calibration)
        if self.constants.reset_calibration_points:
            self.reset_calibration_points()
                   
        # Set sampling frequency
        if self.set_sampling_freq_allowed:
            self.set_speed_mode(self.constants.sampling_freq)
                
        # Enable high performance mode
        if self.enable_processor_high_performance:
            self.enable_processor_high_performance_mode()
        
        # Select RED geometry profile
        if self.geom_profile:
            self.select_RED_geometry(self.constants.geom_profile)
        
        # Get system info 
        self.system_info = self.get_system_info()
        self.Fs = self.system_info['samplerate']
               
        # Quit if sampling rate is not the desired one
        if self.Fs != self.constants.sampling_freq:
            print('Sampling rate other than desired')
            core.quit()
            
        # Check that iview X agree that the desired eye tracker is connected
        eye_tracker_name_iview = self.system_info['iV_ETDevice']
        if ET_device_dict[self.eye_tracker_name] != eye_tracker_name_iview:
            print('Warning: iview thinks that you are using another eye tracker than the one you specified')        
             
        # Internal variable to keep track of whether samples 
        # are put into the buffer or not
        self.__buffer_active = False 
        
        # Remove temp idf-files (otherwise the iview
        # server complains that there are unsaved data)
        self.delete_temp_idf_file()
            
        if self.set_tracking_mode_allowed:
            self.set_tracking_mode(self.constants.track_mode)
            
        # Turn on of off data averaging
        self.average_data(average=self.constants.average_data)
        
        # Never allow the SDK to take control of the calibration keys
        self.set_use_calibration_key(0)
        
        # Enable of disable filters?
        if self.constants.filtering:
            self.enable_bilateral_filter()
        else:
            self.disable_bilateral_filter()
                        
        # Setup calibration parameters
        self.setup_calibration_parameters(autoaccept=self.constants.autoaccept,
                                          cal_method=self.constants.n_cal_points, 
                                          cal_speed=self.constants.cal_speed,
                                          screen = self.constants.screen)
     
    #%% 
    def abort_calibration(self):        
        ''' Aborts calibration
        All system supported
        '''
        res = iViewXAPI.iV_AbortCalibration()
        HandleError(res)    
        
    #%% 
    def abort_calibration_point(self):        
        ''' Aborts calibration point
        Supported systems: REDn, RED250 Mobile
        '''
        res = iViewXAPI.iV_AbortCalibrationPoint()
        HandleError(res)  

            #%%             
    def accept_calibration_point(self):
        ''' Wait for accept 
        All system supported
        '''        
        print('cal_point_accepted')
        res = iViewXAPI.iV_AcceptCalibrationPoint()
        HandleError(res)
        
    #%%
    def calibrate_iview(self):
        ''' Initiate calibration. Calibration parameters first need to 
        be set, see 'setup_calibration_parameters'
        All system supported        
        ''' 
        res = iViewXAPI.iV_Calibrate()
        HandleError(res)       
        
    #%%                     
    def change_calibration_point(self, number, positionX, positionY):
        ''' Change calibration point 'number' to a new position (positionX, positionY)
        All system supported (WARNING: should not be done on the remotes unless
        you REALLY know what you're doing. So don't do it.)       
        This has to be done before the calibration process is started.
        
        Args:
            number - calibration point number (int)
            positionX - x position of new point (pixels)
            positionY - y position of new point (pixels)
            
        Origo of coordinate system is the uppler left corner of screen.
        '''
        res = iViewXAPI.iV_ChangeCalibrationPoint(number, positionX, positionY)
        HandleError(res)              
        
    #%%
    def clear_aoi(self):
        ''' Removes all trigger AOIs
        Not supported. Use your own code and data from the buffer instead
        Supported systems: RED, RED-m, HiSpeed 
        ''' 
        
        res = iViewXAPI.iV_ClearAOI() 
        HandleError(res)        
        
    #%%             
    def clear_recording_buffer(self):
        ''' Clears recording buffer from all recorded data
        Supported systems: all
        '''         
        res = iViewXAPI.iV_ClearRecordingBuffer() 
        HandleError(res)
        
        
    #%%             
    def configure_filter(self, filter_type=0, filter_action=1):
        '''
        Queries or sets filter parameters. The usage of the parameter data depends on the parameter action
        Args: filter_type: 0 - averaging disabled, 1 - averaging enabled
              filter_action: 0 - query the current filter status (output passed to variable 'filter_status'), 
                              1 - configure filter parameters
        Supported systems: all but REDn
        
        WARNING: For some reason, the only combination that works is filter_type=0, filter_action=1, 
        which means that avaraging is disabled. On the other hand, this is not problem. If you want averaged data, just
        average data from the left and the right eye yourself.
        Let me know if you know how to fix it.

        Returns: 
              filter_status - outputs the current filter status (does not work)
        '''
        
        filter_status = c_void_p(0)             
        res = iViewXAPI.iV_ConfigureFilter(c_int(filter_type), 
                c_int(filter_action), byref(filter_status))
        HandleError(res)
        
        #print(filter_type, filter_action, res)
        
        return filter_status

        
    #%%     
    def connect(self, ip_listen, port_listen, ip_send, port_send, 
                connect_timeout=30):
        ''' Connect to eye tracker server
        Supported systems: all 
        
        Args:
            ip_listen       - listen ip address
            port_listen     - list port number
            ip_send         - send ip address
            port_send       - send port number
            connect_timeout - keep trying to connect for 'connect_timeout' s
        '''
        connected = False
        self.clock.reset()
        while not connected and self.clock.getTime() < connect_timeout:
            
            if not ip_send == ip_listen:
                res = iViewXAPI.iV_Connect(c_char_p(ip_listen.encode('ascii')),
                                           c_int(port_listen),
                                           c_char_p(ip_send.encode('ascii')),
                                           c_int(port_send))
            else:
                res = iViewXAPI.iV_ConnectLocal()
                if res != 1:
                    res = iViewXAPI.iV_Connect(c_char_p(ip_listen.encode('ascii')),
                                               c_int(port_listen),
                                               c_char_p(ip_send.encode('ascii')),
                                               c_int(port_send))                  
                                                        
            # If eye tracker is not started, start it and try to connect again
            if res != 1:
                
                HandleError(res)
                #print(ET_server_dict[self.et_server_name])
                try:
                    # Start eye tracker server
                    self.start_iview_server(ET_server_dict[self.et_server_name])
                except:
                    print('Failed to connect. Trying again')
                    
                core.wait(10)
            elif res == 1:
                connected = True
        
        if not connected:
            print('Connection to eye tracker failed after repeated attempts to connect')
            core.quit()  
        
    #%% 
    def continue_eye_tracking(self):
        '''
        Wakes up and enables the eye tracking application from suspend mode to continue processing gaze
        data. The application can be set to suspend mode by calling iV_PauseEyetracking        
        
        Supported systems: all but RED and HiSpeed

        '''
        res = iViewXAPI.iV_ContinueEyeTracking()
        HandleError(res)        
        
    #%% 
    def continue_recording(self, msg):
        '''
        Continues gaze data recording. iV_ContinueRecording does not return until gaze recording is continued.
        Before it can be continued, the data needs to be paused using. iV_PauseRecording. Additionally this
        function allows a message to be stored inside the idf data buffer.        
        
        Supported systems: all
        '''       
        
        res = iViewXAPI.iV_ContinueRecording(msg)
        HandleError(res)              
        
    #%% 
    def define_aoi(self, aoi_data):
        '''
        Defines an AOI. The API can handle up to 20 AOIs
        
        Supported systems: all but RED-n and RED250 mobile.
        
        Args:
            aoi_data - struct, see SDK manual for description
        '''     
        
        res = iViewXAPI.iV_DefineAOI(aoi_data)
        HandleError(res)          

    #%% 
    def define_aoi_port(self, port):
        '''
        Selects a port for sending out TTL trigger
        
        Supported systems: all but RED-n and RED250 mobile.
        
        Args:
            port - int
        '''  
        
        res = iViewXAPI.iV_DefineAOIPort(c_int(port))
        HandleError(res)             
        
    #%% 
    def delete_red_geometry(self, profile):
        '''
        Deletes the geometry setup with the given profile name. It is not possible 
        to delete a geometry profile if it is currently in use. 
        See chapter Setting up RED Geometry in the iView X SDK Manual.
        
        Supported systems: all but HiSpeed
        
        Args:
            profile - string with profile name
        '''              
        res = iViewXAPI.iV_DeleteREDGeometry(profile)
        HandleError(res)
        
    #%% 
    def disable_aoi(self, aoi_name):
        '''
        Disables all AOIs with the given name.
        
        Supported systems: all but RED-n and RED250 mobile.
        
        Args:
            port - int
        '''  
        
        res = iViewXAPI.iV_DisableAOI(c_char(aoi_name))
        HandleError(res)             
        
    #%% 
    def disable_aoi_group(aoi_group):
        '''
        Disables an AOI group
        Supported systems: all but RED-n and RED250 mobile.
        
        Args:
            port - int
        '''  
        
        res = iViewXAPI.iV_DisableAOIGroup(c_char(aoi_group))
        HandleError(res)          
        
    #%% 
    def disable_gaze_data_filter(self):
        '''
        Disables the raw data filter. The gaze data filter can be enabled using 
        iV_EnableGazeDataFilter.        
        Supported systems: all 

        '''   
        res = iViewXAPI.iV_DisableGazeDataFilter()
        HandleError(res)
        
      
        
    #%% 
    def disable_processor_high_performance_mode(self):
        '''
        Disables a CPU high performance mode allowing the CPU to reduce the performance.
        Supported systems: all but RED and Hi-Speed
        '''
        res = iViewXAPI.iV_DisableProcessorHighPerformanceMode()
        HandleError(res)	           
  
    #%% 
    def disconnect(self):
        ''' Disconnects the eye tracker 
        Supported systems: all
        '''
        res = iViewXAPI.iV_Disconnect() 
        HandleError(res)    
        
    #%% 
    def enable_aoi(self, aoi_name):
        '''
        Enables all AOIs with the given name
        Supported systems: all but RED-n and RED250 mobile.
        
        Args:
            aoi_name - string with name of aoi
        '''  
        
        res = iViewXAPI.iV_EnableAOI(c_char(aoi_name))
        HandleError(res)             
        
    #%% 
    def enable_aoi_group(self, aoi_group):
        '''
        Disables an AOI group
        Supported systems: all but RED-n and RED250 mobile.
        
        Args:
            aoi_group - string with name of aoi group
        '''
        
        res = iViewXAPI.iV_EnableAOIGroup(c_char(aoi_group))
        HandleError(res)   
        
    #%%                         
    def enable_gaze_data_filter(self):
        '''
        This API bilateral filter was implemented due to special human-computer
        interaction (HCI) application requirements. It smoothes gaze position data in EyeDataStruct::gazeX and
        EyeDataStruct::gazeY contained in SampleStruct, e.g. obtained by iV_GetSample. The gaze data filter
        can be disabled using iV_DisableGazeDataFilter   
        '''
        res = iViewXAPI.iV_EnableGazeDataFilter() 
        HandleError(res)   
        
        
    #%% 
    def enable_processor_high_performance_mode(self):
        '''
        Enables a CPU high performance mode allowing the CPU to reduce the performance.
        Supported systems: all but RED and Hi-Speed
        '''
        res = iViewXAPI.iV_EnableProcessorHighPerformanceMode()
        HandleError(res)	    
        
    #%%
    def get_accuracy(self, visualization = 0):
        ''' Get accuracy. Only possible after a successful validation
        If the parameter visualization is set to 1 the accuracy
        data will be visualized in a dialog window.
        
        Args:
            visualization - int
            
        Returns:
            accuracy values for left and right eyes
            
        ''' 

        res = iViewXAPI.iV_GetAccuracy(byref(accuracyData), visualization)
        HandleError(res)	
        
        return (accuracyData.deviationLX, accuracyData.deviationLY,
               accuracyData.deviationRX, accuracyData.deviationRX)

    #%%         
    def get_accuracy_image(self, fname=None):
        ''' Returns validation screen image and optinally save it to disk
        
        Args:
            fname - name of image to be saved to disk, e.g., 'im.png'
            
        Returns:
            im - image as n x m x 3 numpy array
        '''
        
        # update imageData with the most recent accuracy image        
        res = iViewXAPI.iV_GetAccuracyImage(byref(imageData))
        
        # Convert imageData.imageBuffer to something understandable
        ac = np.array(imageData.imageBuffer[:imageData.imageSize], 'c')
        ac_as_int = ac.view(np.uint8)
        ac_as_int = ac_as_int[:imageData.imageSize]
        
        # Make background gray instead of black
        ac_as_int[ac_as_int == 0] = 128
        im = np.reshape(ac_as_int, [imageData.imageHeight, imageData.imageWidth, 3]) 
        
        # Save image to disk if a file name is given
        if fname:
            misc.imsave(fname, im)
            core.wait(0.1)
            
        return im
        
    #%% 
    def get_aoi_output_value(self):
        '''
        Returns the current AOI value.
        Supported systems: all 
        
        Returns:
            aoiOutputValue - int
        '''  
        
        res = iViewXAPI.iV_getAOIOutputValue(byref(aoiOutputValue))
        HandleError(res)     
        
        return aoiOutputValue
        
    #%%
    def get_calibration_parameter(self):
        ''' Updates stored calibrationData information with currently selected 
            parameters.
        Supported systems: RED-n and RED250 Mobile 
        
        Returns:
            calibrationData - structure containing information about calibration
        '''
        
        res = iViewXAPI.iV_getCalibrationParameter(byref(calibrationData))
        HandleError(res)           
        return calibrationData
        
    #%%
    def get_calibration_point(self, calibration_point_number):
        ''' Delivers information about a calibration point.
        Supported systems: all
        
        Args:
            calibration_point_number - number of calibration point
        
        Returns:
            calibrationPoint - struct with info about calibration point
            Contains number (int), positionX (int), and positionY (int)
        '''
        
        res = iViewXAPI.iV_getCalibrationPoint(c_int(calibration_point_number),
                                               byref(calibrationPoint))
        HandleError(res)           
        return calibrationPoint   
        
    #%%
    def get_calibration_quality(self, calibration_point_number):
        ''' Delivers fixation quality information about a calibration point. 
        If the passed parameter left or right is NULL, no data will be returned
        Supported systems: RED-n and RED250 Mobile 
        
        Args:
            calibration_point_number - number of calibration point        
        '''      
        
        res = iViewXAPI.iV_getCalibrationPointQuality(c_int(calibration_point_number),
                                               byref(left), byref(right))
        HandleError(res)           
        return left, right          
        
    #%%
    def get_calibration_quality_image(self):
        ''' 
        Same functionally as get_accuracy_image
        Supported systems: RED-n and RED250 Mobile 

        Returns:
            imageData - 
        '''   
        
        res = iViewXAPI.iV_getCalibrationPoint(byref(imageData))
        HandleError(res)           
        return imageData           
        
    #%%
    def get_calibration_status(self):
        ''' Updates calibrationStatus information. 
        The client needs to be connected to the iView eye tracking server.
        Supported systems: all        
        ''' 
        
        res = iViewXAPI.iV_getCalibrationStatus(byref(calibrationStatus))
        HandleError(res)           
        return calibrationStatus         

    #%%
    def get_current_calibration_point(self):
        ''' Updates data in currentCalibrationPoint with the current calibration 
        point position
        Supported systems: all        
        '''
        res = iViewXAPI.iV_GetCurrentCalibrationPoint(byref(currentCalibrationPoint))
        HandleError(res)       
        
        return res, currentCalibrationPoint

    #%% 
    def get_current_RED_geometry(self):
        '''
        Supported systems: all but HiSpeed
        '''        
        res = iViewXAPI.iV_GetCurrentREDGeometry(byref(redGeometry))
        HandleError(res)
                    
        return redGeometry
    
    #%% 
    def get_current_time_stamp(self):
        ''' Provides the current eye tracker timestamp in microseconds
        Supported systems: all
        '''        
        res = iViewXAPI.iV_GetCurrentTimestamp(byref(currentTimestamp))
        HandleError(res)
                    
        return currentTimestamp  
    
    #%%
    def get_device_name(self):
        ''' Queries the device name information of the connected device.
        Supported systems: all but RED and HiSpeed
        '''                
        res = iViewXAPI.iV_GetDeviceName(byref(deviceName))
        HandleError(res)
        
        return deviceName    
    
    #%%
    def get_event(self):
        ''' Updates data from eventDataSample with current event data.
        Supported systems: all but RED-n professional
        '''                
        res = iViewXAPI.iV_GetEvent(byref(eventDataSample))
        HandleError(res)
        
        return eventDataSample        
    
    #%%                     
    def get_eye_image(self):
        ''' Updates imageData with current eye image (format: monochrome 8bpp).
        Supported systems: ToDo
        '''   
        res = iViewXAPI.iV_GetEyeImage(byref(imageData)) 
        #core.wait(0.)
        if res == 1:
            
            # Convert image to 1-d array
            ac = np.array(imageData.imageBuffer[:imageData.imageSize], 'c')
            ac_as_int = ac.view(np.uint8)
            ac_as_int = ac_as_int[:imageData.imageSize]
            
            # Reshape to 2-d image and normalize values to [-1, 1]
            im = np.zeros(imageData.imageSize)
            im[:np.shape(ac_as_int)[0]] = ac_as_int
            im = np.reshape(im, [imageData.imageHeight, imageData.imageWidth])
            
            #np.save('eye_im', im)
            
            im = np.fliplr((im / float(im.max()) * 2) - 1)
            #print(imageData.imageHeight, imageData.imageWidth, 3)

            im_res = [imageData.imageHeight, imageData.imageWidth]
            
        else:
            im_res = [self.constants.eye_image_size[0], self.constants.eye_image_size[1]]
            im = np.zeros([self.constants.eye_image_size[0], self.constants.eye_image_size[1]])
            HandleError(res)


            
        # Fit image to a 512x512 container (must be power of 2)
        # 
        if self.constants.eye_tracker_name == 'HiSpeed':
            im_sz = 1024
        else:
            im_sz = 512
            
        # Scale the eye image
        #print(imageData.imageHeight, imageData.imageWidth, np.shape(im))
        im_final = np.zeros([im_sz, im_sz])
        row_idx = (im_sz - im_res[0]) / 2
        col_idx = (im_sz - im_res[1]) / 2
        im_final[int(row_idx):int(row_idx+im_res[0]),
                 int(col_idx):int(col_idx+im_res[1])] = np.rot90(im, 2)
        
        return im_final, res

    #%%
    def get_feature_key(self):
        ''' Gets the device specific feature key. Used for RED-OEM, RED250mobile and REDn devices only
        Supported systems: RED-n and RED250 Mobile
        '''                
        
        res = iViewXAPI.iV_GetFeatureKey(byref(featureKey))
        HandleError(res)
        
        return featureKey        

    #%%
    def get_gaze_channel_quality(self):
        ''' Retrieve gaze quality data. Fills qualityData with validated accuracy results. Before quality data is
        accessible the system needs to be validated with iV_Validate
        Supported systems: RED-n and RED250 Mobile
        
        ''' 
        
        res = iViewXAPI.iV_GetGazeChannelQuality(byref(qualityData))
        HandleError(res)
        
        return qualityData            
        
        
    #%%
    def get_recording_state(self):
        ''' Queries the recording state of the eye tracking server. 
        This function can be used to check if the eye
        tracking server is currently performing a recording.
        Supported systems: RED-n and RED250 Mobile
        '''             
        
        res = iViewXAPI.iV_GetRecordingState(byref(recordingState))
        HandleError(res)
        
        return recordingState

    #%% 
    def get_RED_geometry(self, profile_name):
        ''' Gets the geometry data of a requested profile without selecting them.
        Supported systems: all but HiSpeed
        '''                
        res = iViewXAPI.iV_GetREDGeometry(profile_name, byref(redGeometry))
        HandleError(res)
                    
        return redGeometry    


    #%%             
    def get_sample(self):
        ''' Updates data in sampleData with current eye tracking data.
        Supported systems: all
        ''' 
        res = iViewXAPI.iV_GetSample(byref(sampleData))
        HandleError(res)
        return sampleData 
    
    #%%
    def get_scene_video(self):
        ''' Updates imageData with current scene video image (format: RGB 24bpp)
        Not Supported 
        '''     
        
    #%%
    def get_serial_number(self):
        ''' Retrieve the serial number information of the connected device.
        Supported systems: all but RED and HiSpeed
        '''       
        
        res = iViewXAPI.iV_GetSample(byref(serialNumber))
        HandleError(res)
        return serialNumber     
    
    #%%
    def get_speed_mode(self):
        ''' This function retrieves the speed modes used and supported by the 
        connected iView eye tracking server
        
        speedModes:
        int numberOfSpeedModes - number of supported speed modes
        int speedMode - the current sampling frequency
        int speedModes - an array of sampling frequencies supported by the connected iView eye tracking server;
        int version - version of the current data structure        
        Supported systems: RED-n and RED250 Mobile
        '''       
        
        res = iViewXAPI.iV_GetSpeedModes(byref(speedModes))
        HandleError(res)
        return  speedModes       
    
    #%%             
    def get_system_info(self):
        '''
        int API_Buildnumber build number of iView X SDK in use
        int API_MajorVersion - major version number of iView X SDK in use
        int API_MinorVersion - minor version number of iView X SDK in use
        int iV_Buildnumber - build number of iView eye tracking server in use
        enum ETDevice- 	iV_ETDevice type of eye tracking device
        int iV_MajorVersion - major version number of iView eye tracking server in use		
        int iV_MinorVersion - major version number of iView eye tracking server in use		
        int samplerate
        
        ETDevice {
        NONE = 0, RED = 1, REDm = 2, HiSpeed = 3,
        MRI = 4, HED = 5, Custom = 7, REDn = 8 }	
        
        Supported systems: all        
        '''
    
        res = iViewXAPI.iV_GetSystemInfo(byref(systemData))
        HandleError(res)
        
        system_info = {'API_Buildnumber':systemData.API_Buildnumber,
        'API_Buildnumber':systemData.API_Buildnumber,
        'API_MajorVersion':systemData.API_MajorVersion,
        'API_MinorVersion':systemData.API_MinorVersion,
        'iV_Buildnumber':systemData.iV_Buildnumber,
        'iV_ETDevice':systemData.iV_ETDevice,
        'iV_MajorVersion':systemData.iV_MajorVersion,
        'iV_MinorVersion':systemData.iV_MajorVersion,
        'samplerate':systemData.samplerate}
        print('hej')
        print(systemData.iV_ETDevice)
        return system_info  
    
    #%%     
    def get_tracking_mode(self):
        ''' Get eye tracking mode (see set_tracking_mode)
        ''' 
        res = iViewXAPI.iV_GetTrackingMode(byref(mode))
        HandleError(res)
        
        return mode    
    
    #%%         
    def get_tracking_monitor(self):
        ''' Returns tracking monitor
        
        The tracking monitor image depicts the positions of both eyes and shows notification arrows 
        if the participant is not properly positioned infront of the eye tracker. 
        The tracking monitor is useful to validate the positioning before and 
        during a recording session.  
        
        Supported systems: all but HiSpeed                   
        '''
        
        # update imageData with the most recent accuracy image        
        res = iViewXAPI.iV_GetTrackingMonitor(byref(imageData))
        
        # Convert imageData.imageBuffer to something understandable
        ac = np.array(imageData.imageBuffer[:imageData.imageSize], 'c')
        ac_as_int = ac.view(np.uint8)
        ac_as_int = ac_as_int[:imageData.imageSize]
        #print(imageData.imageHeight, imageData.imageWidth)
        
        # ValueError: total size of new array must be unchanged
        if len(ac_as_int) == imageData.imageHeight * imageData.imageWidth * 3:
            im = np.reshape(ac_as_int, [imageData.imageHeight, imageData.imageWidth, 3])
            im = (im / float(im.max()) * 2) - 1
            im = im[:, :, ::-1]
            im[im < 0] = 0
            im[0, 0, 0] = -1
            im = np.fliplr(im)
        else:
            im = np.zeros((self.constants.eye_image_size[0], self.constants.eye_image_size[1], 3))
            res = 2

        return im, res   
    
    #%% 
    def get_tracking_status(self):
        ''' Updates trackingStatus with current tracking status.
        This function can be used to get the current eye positions.
        
        Supported systems: all 
        
        '''
        res =  iViewXAPI.iV_GetTrackingStatus(byref(trackingStatus))
        HandleError(res)
        return trackingStatus      
    
    #%%
    def get_use_calibration_key(self):
        ''' Gets the currently set interaction key status for the calibration and validation process. 
        If enableKeys is 0 all available user interaction keys:
            • SPACE for accepting calibration/validation points
            • ESC for aborting calibration/validation
            • TAB for skipping a point (only SMI iViewRED 4.2 or later)
        are disabled.
        
        Supported systems: RED-n and RED250 Mobile 
        '''
        
        enableKeys = c_int(0)
        res = iViewXAPI.iV_GetUseCalibrationKeys(byref(enableKeys))
        HandleError(res)
        return enableKeys   
    
    
    #%%
    def hide_accuracy_monitor(self):
        ''' Hides accuracy monitor window which can be opened by iV_ShowAccuracyMonitor.
        Supported systems: all
        '''     
        
        res = iViewXAPI.iV_HideAccuracyMonitor()
        HandleError(res)        
        
    #%%
    def hide_eye_image_monitor(self):
        ''' Hides eye image monitor window which can be opened by iV_ShowEyeImageMonitor.
        Supported systems: all but RED-n professional
        '''     
        
        res = iViewXAPI.iV_HideEyeImageMonitor()
        HandleError(res)         

    #%%
    def hide_scene_video_monitor(self):
        ''' Hides scene video monitor window which can be opened by iV_ShowSceneVideoMonitor.
        Not Supported 
        '''   

                #%%
    def hide_tracking_monitor(self):
        ''' Hides tracking monitor window which can be opened by iV_ShowTrackingMonitor
        Supported systems: all but HiSpeed
        '''  
        
        res = iViewXAPI.iV_HideTrackingMonitor()
        HandleError(res)           
        
    #%% 
    def is_connected(self):
        ''' Checks if connection to iView eye tracking server is still established.
        Supported systems: all 
        
        Returns:
            res - 1 if intended functionality has been fulfilled
                  0 if no connection established
        
        '''
        
        res = iViewXAPI.iV_IsConnected()
        HandleError(res)
        return res
    
    #%%         
    def load_calibration(self, name):
        ''' Loads a previously saved calibration. A calibration has to be saved by using iV_SaveCalibration.
        
        Supported systems: all 
        
        '''        
        res = iViewXAPI.iV_LoadCalibration(c_char_p(name.encode('ascii')))
        HandleError(res)    
        
    #%%         
    def log(self, msg):
        ''' Writes logMessage into log file
        
        Supported systems: all 
        
        '''        
        res = iViewXAPI.iV_Log(c_char_p(msg.encode('ascii')))
        HandleError(res)       
        
    #%%
    def pause_eye_tracking(self):
        ''' Suspend the eye tracking application and disables calculation of gaze data. 
        The application can be reactivated by calling iV_ContinueEyetracking.
        Supported systems: all but RED and HiSpeed
        '''     
        
        res = iViewXAPI.iV_PauseEyeTracking()
        HandleError(res)          
        
    #%%
    def pause_recording(self):
        ''' Pauses gaze data recording. iV_PauseRecording does not return until 
        gaze recording is paused.
        Supported systems: all 
        '''    
        
        res = iViewXAPI.iV_PauseRecording()
        HandleError(res)         
        
    #%% 
    def quit_server(self):
        ''' Disconnects and closes iView eye tracking server. 
        After this function has been called no other function
        or application can communicate with iView eye tracking server.
        '''
        
        res = iViewXAPI.iV_Quit()  
        HandleError(res)
        
    #%% 
    def recalibrate_one_point(self, calibration_point_number):
        ''' Restarts a calibration procedure with a point from the latest calibration process. 
        The point is specified
        by its index in the calibration point profile (counted from 1). If the 
        requested point is not found, an error
        code will be returned. The number of calibration points can be retrieved 
        via iV_GetCalibrationQuality
        
        Supported systems: RED-n and RED250 Mobile 
        '''
        
        res = iViewXAPI.iV_RecalibrateOnePoint(c_int(calibration_point_number))  
        HandleError(res)        
        
        
    #%%
    def release_aoi_port(self):
        ''' Releases the port for sending TTL trigger.
        Supported systems: all but RED-n and RED250 Mobile 
        '''
        
        res = iViewXAPI.iV_ReleaseAOIPort()  
        HandleError(res)            
        
    #%%
    def remove_aoi(self, name):
        ''' Removes all AOIs with the given name.
        Supported systems: all but RED-n and RED250 Mobile 
        '''   

        res = iViewXAPI.iV_RemoveAOI()  
        HandleError(res)    
        
    #%%                         
    def reset_calibration_points(self):           
        ''' Resets the positions of the calibration points
        Supported systems: all 
        
        '''         
        res = iViewXAPI.iV_ResetCalibrationPoints()
        HandleError(res)        
        
    #%%    	
    def save_calibration(self, name):
        ''' Saves a calibration with a custom name. To save a calibration it 
        is required that a successful calibration already has been completed.
        
        Supported systems: all 
        
        '''
        res = iViewXAPI.iV_SaveCalibration(c_char_p(name.encode('ascii')))
        HandleError(res)      
        
    #%%             
    def save_data(self, filename, description = "", 
                   user = None, append_version=True):
        ''' Writes recorded data buffer to disc. 
        The data recording needs to be stopped using iV_StopRecording
        before the data buffer can be saved to given location. 
        The filename can include the path. If the connected eye tracking device 
        is an HED, scene video buffer is written, too. iV_SaveData will not return
        until the data has been saved.
        
        If there is already a file with a certain name 'name.idf', this file will not 
        be overwritten, but save as another file with name 'name_1.idf'.
        
        Args:
            filename - full path including the filename of the data file being created
            description - Optional experiment description tag stored in the idf file. This tag is available in BeGaze and in the text export from an idf file.
            user - Optional name of test person. This tag is available in BeGaze and in the text export
                    from an idf file.
            append_version - append version number to file if exists (e.g., _1, _2, etc)
        
        Supported systems: all 
        
        '''
                    
        # Set the use equal to the filename if not explicitly given
        if user == None:
            user = filename
            
        # Split filename into path and filename
        path, filename = os.path.split(filename)
        assert(len(path) > 0), "Filename must have a path"
        assert(len(filename) > 0), "Filename must be given"

        # Check if a '.idf was added to the filename. If so, remove it
        ext = os.path.splitext(filename)[1]
        if '.idf' in ext:
            filename = filename.strip('.idf')
            
        # If the filename already exists, append _x
        files = glob.glob(path + os.sep + '*.idf')
        
        i = 1
        filename_ext = ''
        while True:
            
            # Go through the files and look for a match
            filename_exists = False
            for f in files:
                f_temp = f.split('\\')[-1][:-4]
                
                # if the file exists
                if filename + filename_ext == f_temp:
                    if not append_version:
                        raise ValueError('Warning! Filename already exists')
                    else: # append '_i to filename
                        filename_ext = '_' + str(i)          
                        filename_exists = True
                        i += 1
                    
            # If we've gone through all files without
            # a match, we ready!
            if not filename_exists:
                 break
             
        # Add the new extension the the filename        
        filename = os.sep.join([path, filename + filename_ext + ext])
        
        # If two computer setup (TODO)
#        if not ip_send == ip_listen:
#            # Two computer setup: file gets saved on eye-tracker
#            # computer (do so with without path info and allowing
#            # overwrite). Transfer the file using the
#            # FileTransferServer running on the remote machine. NB:
#            # this seems to only work when iView is running on the
#            # remote machine.
#            
#            # 1: connect to file transfer server
#            # 1a: request FileTransferServer.exe's version (always
#            #     happens when experiment center is just started)            
#            pass
#        else:
#            pass
#            print(filename)
        res = iViewXAPI.iV_SaveData(c_char_p(filename.encode('ascii')), 
                                    c_char_p(description.encode('ascii')), 
                                    c_char_p(user.encode('ascii')), 
                                    0) # Never overwrite existing file with the same name
        HandleError(res)       
        
    #%% 
    def select_RED_geometry(self, profile):
        ''' Selects a predefined geometry profile. 
        
        Supported systems: all but HiSpeed 
        
        '''
        print(profile)
        res = iViewXAPI.iV_SelectREDGeometry(c_char_p(profile.encode('ascii')))
        HandleError(res)   
        assert(res==1), "RED geometry profile does not exist"
        
        
    #%%
    def send_command(self, cmd):
        ''' Sends a remote command to iView eye tracking server. 
        Please refer to the iView X help file for further information about remote commands.
        
        Supported systems: all 
        '''
        res = iViewXAPI.iV_SendCommand(cmd)
        HandleError(res)       
        
    #%%             
    def send_image_message(self, msg):
        ''' Sends a text message to iView X idf recording data file. 
        If the etMessage has the suffix ".jpg", ".bmp",
        ".png", or ".avi" BeGaze will separate the data buffer 
        automatically into according trials.
        
        Supported systems: all 
        '''
        res = iViewXAPI.iV_SendImageMessage(c_char_p(msg.encode('ascii')))
        HandleError(res)    
        
    #%%
    def set_aoi_hit_callback(self, callback_function):
        ''' Sets a callback function for the AOI hit functions.
        Supported systems: all but RED-n and RED250 Mobile
        
        '''
        res = iViewXAPI.iV_SetAOIHitCallback(callback_function)
        HandleError(res)         
        
    #%%
    def set_calibration_callback(self, callback_function):
        ''' Sets a callback function for the AOI hit functions.
        Supported systems: All
        '''           
        
        res = iViewXAPI.iV_SetCalibrationCallback(callback_function)
        HandleError(res)              
        
    #%%
    def set_connection_timeout(self, time):
        ''' Defines a customized timeout for how long iV_Connect tries to 
        connect to iView eye tracking server.
        Supported systems: all but RED-n professional 
        '''          
        
        res = iViewXAPI.iV_SetConnectionTimeout(time)
        HandleError(res)   

    #%%
    def set_event_callback(self, callback_function):
        ''' Sets a callback function for the event data. 
        The function will be called if a real-time detected fixation has
        been started or ended.
        Supported systems: all but RED-n professional 
        '''          
        
        res = iViewXAPI.iV_SetEventCallback(callback_function)
        HandleError(res)     
        
    #%%
    def set_event_detection_parameters(self, name):
        ''' Defines the detection parameter for online fixation detection algorithm.
        Supported systems: all but RED-n professional 
        '''          
        
    #%%
    def set_eye_image_callback(self, callback_function):
        ''' Sets a callback function for the eye image data. 
        Supported systems: all but RED-n professional and RED-mx
        '''     
        
        res = iViewXAPI.iV_SetLicense(callback_function)
        HandleError(res)     
        
    #%%
    def set_licence(self, key):
        ''' Sets the customer license (required only for OEM devices!).
        Supported systems: RED-n and RED-n scientific
        '''             
        
        res = iViewXAPI.iV_SetLicense(key)
        HandleError(res)     
        
    #%%
    def set_logger(self, log_level=1, filename='iv_logfile'):
        ''' Sets the customer license (required only for OEM devices!).
        
        ToDo: What log levels are there and what do they mean?
        Supported systems: all 
        
        '''             
        
        res = iViewXAPI.iV_SetLogger(key)
        HandleError(res)        
        
    #%%
    def set_resolution(self, stimulus_width, stimulus_height):
        ''' Sets the customer license (required only for OEM devices!).
        
        Defines a fixed resolution independent to the screen resolution of 
        chosen display device defined in iV_-SetupCalibration function.
        
        Could be useful when using real-time data with a screen with low resolution.
        Supported systems: all 
        
        '''             
        
        res = iViewXAPI.iV_SetResolution (c_int(stimulus_width), 
                                          c_int(stimulus_height))
        HandleError(res)   
        
    #%%
    def set_RED_geometry(self, setup_mode = 0, 
                                    monitor_size = 22,
                                    setup_name = 'test',
                                    stim_x=0,
                                    stim_y=0,
                                    stim_height_over_floor=0,
                                    red_height_over_floor=0,
                                    red_stim_dist=0,
                                    red_incl_angle=0,
                                    red_stim_dist_height=0,
                                    red_stim_dist_depth=0):
        ''' Define the eye trackers stand alone and monitor integrated geometry
        
        The dict should included
        
        Supported systems: all but HiSpeed
        
        int monitorSize:        monitor size [inch] can be set to 19 or 22 used if redGeometry is 
                                set to monitorIntegrated only
        enum REDGeometryEnum redGeometry: defines which parameter is used. Can be 'monitorIntegrated' (0) or 'standalone'(1)
        
        int redHeightOverFloor: distance floor to eye tracking device [mm] used 
                                if redGeometry is set to standalone only
        int redInclAngle:       eye tracking device inclination angle [degree] used if redGeometry
                                is set to standalone only
        int redStimDist:        distance eye tracking device to stimulus screen [mm] used if red-
                                Geometry is set to standalone only
        int redStimDistDepth:   horizontal distance eye tracking device to stimulus screen [mm]
                                used if redGeometry is set to standalone only
        int redStimDistHeight:  vertical distance eye tracking device to stimulus screen [mm] used
                                if redGeometry is set to standalone only
        char setupName:         name of the profile used if redGeometry is set to standalone only
        int stimHeightOverFloor:distance floor to stimulus screen [mm] used if redGeometry is set
                                to standalone only
        int stimX:              horizontal stimulus calibration size [mm] used if redGeometry is
                                set to standalone only
        int stimY:              vertical stimulus calibration size [mm] used if redGeometry is set
                                to standalone only        
                                    
        '''      
        
        redGeometry = CREDGeometry(setup_mode, 
                                    monitor_size,
                                    setup_name,
                                    stim_x,
                                    stim_y,
                                    stim_height_over_floor,
                                    red_height_over_floor,
                                    red_stim_dist,
                                    red_incl_angle,
                                    red_stim_dist_height,
                                    red_stim_dist_depth)
                                    
        res = iViewXAPI.iV_SetREDGeometry(redGeometry)
        self.geom = self.get_current_RED_geometry()
        HandleError(res)           
        print(res)
    #%%
    def set_sample_callback(self, function_name):
        ''' Sets a callback function for the raw sample data. 
        The function will be called if iView eye tracking server
        has calculated a new data sample.
        Attention: Algorithms with high processor usage and long calculation 
        time should not run within this callback due to a higher probability of data loss
        Supported systems: all 
       
        '''             
        
        res = iViewXAPI.iV_SetSampleCallback(self.function_name)
        HandleError(res)    
        
    #%%
    def set_scene_video_callback(self, name):
        ''' Sets a callback function for the scene video image data. 
        The function will be called if a new scene video
        image is available. The image format is RGB 24bpp.
        Not Supported. 
        '''    

    #%%
    def set_speed_mode(self, samplingrate):
        ''' This function requests the iView eye tracking server to switch the
        eye tracking frequency to the specified value. Use iV_GetSpeedModes 
        to get the available speed modes for the connected eye tracking device.
        Supported systems: RED-n 
        '''        
        
        if self.set_sampling_freq_allowed:
            res = iViewXAPI.iV_SetSpeedMode(c_int(samplingrate))
            HandleError(res) 
        else:
            print("WARNING: set_speed_mode is not supported on this eye tracker")
            
    #%%     
    def set_tracking_mode(self, mode):
        ''' This function is available with SMI iViewRED 4.4 or later and replaces the iV_SetTrackingParameter
            function
            
            e.g., set_tracking_mode(self, 'SMART_BINOCULAR')
        
            Eye tracking modes:
        
            smart_binocular: tracks both eye separately, but can handle temporal monocular loss (default)
            smart_binocular_right/left: both eye visible, but one one dominant (e.g., obvious squinting)
            monocular_right/left: only one eye visible
            
            0 - SmartBinocular SmartBinocular mode.
            1 - MonocularLeft Monocular mode using only the left eye.
            2 - MonocularRight Monocular mode using only the right eye.
            3 - Binocular Binocular mode.
            4 - SmartMonocular SmartMonocular mode.	
            
        Supported systems: all but RED and HiSpeed 
            '''
        if self.set_tracking_mode_allowed:
            assert (mode == 'SMART_BINOCULAR' or 
                    mode == 'MONOCULAR_LEFT' or 
                    mode == 'MONOCULAR_RIGHT' or 
                    mode == 'BINOCULAR' or 
                    mode == 'SMART_MONOCULAR')
            
            res = iViewXAPI.iV_SetTrackingMode(tracking_mode_dict[mode])
            HandleError(res)
        else:
            print("WARNING: set_tracking_mode is not supported on this eye tracker")          
            
    #%%
    def set_tracking_monitor_callback(self, function_name):
        ''' Sets a callback function for the tracking monitor image data. 
        The function will be called if a new tracking
        monitor image was calculated. The image format is BGR 24bpp
        Supported systems:  all but HiSpeed
        '''    

        res = iViewXAPI.iV_SetTrackingMode(function_name)
        HandleError(res)
        
        
    #%%     
    def set_tracking_parameter(self, eye_type=0, parameter_type=4, activate=1):       
        ''' Sets iView eye tracking server tracking parameters. 
        See Eye Tracking Parameter subsection and iView eye tracking server 
        manual for further explanations. Important note: This function can 
        strongly affect tracking stability of your iView X and 
        eyetracking-server system. Only experienced users should use this
        function.
        
        Args:
            eye_type - select specific eye (0 is left, 1 is right)
            parameter_type - parameter to set (see manual)
            activate - new value for selected parameter
            
        Supported systems: ToDo 
        '''          
        res = iViewXAPI.iV_SetTrackingParameter(c_int(eye_type), 
                                                c_int(parameter_type),
                                                c_int(activate))
        HandleError(res)   
        
    #%%     
    def setup_calibration_parameters(self,
                                     autoaccept=1,
                                     bg_color=0,
                                     screen=1,
                                     fg_color=0,
                                     cal_method=5, 
                                     cal_speed=1,
                                     target_size=20, 
                                     target_shape=2): 
                                                            
        """
        Sets the calibration and validation visualization parameter.
        
        Setup calibration parameters (but do not initiate calibration)
        An option to define position of calibration point
        
        1 - autoAccept
        2-  background Brightness
        3-  displayDevice
        4 - foreground Brightness
        5 - cal method
        6 - speed (cal)
        7 - target Filename[256]
        8 - targetShape
        9 - targetSize
        10 - visualization
        
        Supported systems: all 
        
        
        """
        
        calibrationData = CCalibration(cal_method,
                                       0, # Always use Psychopy for visualization
                                       screen,
                                       cal_speed,
                                       autoaccept,
                                       fg_color,
                                       bg_color,
                                       target_shape,
                                       target_size,
                                       b"")
        res = iViewXAPI.iV_SetupCalibration(byref(calibrationData))
        print('CCdata {}'.format(res))
        HandleError(res)        
        
    #%%
    def setup_debug_mode(self, enable_debug_mode=False):
        '''Enables or disables the debug mode for the current connection. 
        The debug mode disables the automatic
        connection termination after 5 seconds of an unresponsive server or client. 
        This can happen e.g. during
        debugging a client application. Beware: the debug mode must not be enabled 
        for production code, as it
        makes the connection status detection of all API functions unreliable!
                
        Supported systems: ?
        '''
        res = iViewXAPI.iV_SetupDebugMode(c_int(enable_debug_mode))
        HandleError(res)            
        
    #%%
    def setup_ltp_recording(self, port_name, enable_recording):
        '''Enables or disables the LPT signal recording functionality. 
        Not Supported. 
        '''     
        
        res = iViewXAPI.iV_SetupLtpRecording(c_char(port_name), c_int(enable_recording))
        HandleError(res)                 
            
    #%%
    def set_use_calibration_key(self, mode):
        ''' Sets and resets the interaction keys during the calibration and validation process.
        See get_use_calibration_key
        '''
        res = iViewXAPI.iV_SetUseCalibrationKeys(c_int(mode))
        HandleError(res)         
        
    #%%
    def show_accuracy_monitor(self):
        '''The validated accuracy results will be visualized in a separate window. 
        Before the image can be drawn the calibration needs to be performed with 
        iV_Calibrate and validated with iV_Validate.
        Supported systems: all
        '''      
        res = iViewXAPI.iV_ShowAccuracyMonitor()
        HandleError(res)            
        
    #%%
    def show_eye_image_monitor(self):
        '''Visualizes eye image in a separate window while the participant is 
        beeing tracked (equal to image obtained with iV_GetEyeImage).
        Supported systems: all but RED-n professional and RED-m mx
        '''     
        
        res = iViewXAPI.iV_ShowEyeImageMonitor()
        HandleError(res)         
        
    #%%
    def show_scene_video_monitor(self):
        '''Visualizes scene video in separate window.
        Only available for HED devices.
        Not Supported. 
        '''           
        
    #%%
    def show_tracking_monitor(self):
        '''Visualizes RED tracking monitor in a separate window.
        Supported systems: all but HiSpeed
        '''     
        
        res = iViewXAPI.iV_ShowTrackingMonitor()
        HandleError(res)       

    #%% 
    def start_iview_server(self, et_application):
        '''Starts the iView eye tracking server application. Depending on the PC, 
        it may take several seconds to start the iView eye tracking server application. 
        The connection needs to be established separately using iV_Connect. 
        The connection timeout can be extended using iV_SetConnectionTimeout.
        
        Supported systems: all 
        '''               
        print(et_application)
        res = iViewXAPI.iV_Start(c_int(et_application))
        HandleError(res)
        
    #%%             
    def start_recording(self):
        ''' Starts gaze data recording
        Supported systems: all
        '''
        res = iViewXAPI.iV_StartRecording()
        HandleError(res)        
        
    #%%             
    def stop_recording(self):
        ''' Stops gaze data recording
        Supported systems: all
        '''
        res = iViewXAPI.iV_StopRecording()
        HandleError(res)     
        
    #%%
    def test_ttl(self, value):
        '''Sends a TTL value to defined port. Define a port with iV_DefineAOIPort
        Supported systems: all
        '''
        res = iViewXAPI.iV_TestTTL(c_int(value))
        HandleError(res)   

    #%%    
    def validate_iview(self):
        ''' Starts a validation procedure. To proceed, the participant needs to 
        be tracked and has to fixate the validation point. 
        Depending on the validation settings (which can be changed using iV_SetupCalibration
        and iV_SetUseCalibrationKeys) the user can accept the validation points 
        manually (by pressing [SPACE] or calling iV_AcceptCalibrationPoint) or 
        abort the validation (by pressing [ESC] or calling iV_AbortCalibration)
        If the validation is visualized by the 
        API (CalibrationStruct::visualization is set to 1) the function will not
        return until the validation has been finished (closed automatically) 
        or aborted (by using [ESC]).
        If the CalibrationStruct::visualization is set to 0, the function call returns immediately. 
        The user has to implement the visualization of validation points. 
        Information about the current validation point can be retrieved with 
        iV_GetCurrentCalibrationPoint or with setting up the calibration callback using
        iV_SetCalibrationCallback.
        ''' 
        res = iViewXAPI.iV_Validate()
        HandleError(res)	
        
    ###############################################################################
    '''
    Below are convenience functions that extends the basic iview 
    functionally about or/and make calls more transparent
    '''
    ###############################################################################        
 
    #%%             
    def set_cal_positions(self, cal_positions):
        """
        Sets the positions of the calibration locations
        cal_positions is a dict:  {1:[x,y],2:[x,y],....}
        """
        if cal_positions:
            for k in cal_positions.keys():
                self.change_calibration_point(k, cal_positions[k][0], cal_positions[k][1])    

     

    #%% 
    def set_begaze_trial_image(self, imname):
        '''
        imname - ex. 'testimage.jpg'
        The filename should not include a path
        '''
        
        # Skip the path if there is one
        filename  = os.path.split(imname)[1]
        
        # Get the file extension
        ext = os.path.splitext(imname)[1]
        
        # check extention is one of the supported ones
        assert(len([i for i in ['.png','.jpg','.jpeg','.bmp','.avi'] if ext == i]) > 0), "Filename not supported"
        self.send_image_message(imname)
        
    #%% 
    def set_begaze_mouse_click(self, which, x, y):
        ''' Make BeGaze understand that a mouse click has happened
        '''
        assert which in 'left' or which in 'right', 'SMITE: SMI BeGaze mouse press must be for ''left'' or ''right'' mouse button'

        self.send_image_message('UE-mouseclick {} x={} y={}'.format(which, x, y))  
        
    #%% 
    def set_begaze_key_press(self, string):
        '''  can use this to send any string into BeGaze event stream (do
        not know length limit). We advise to keep this short
        special format to achieve this
        '''
        self.send_image_message('UE-keypress {}'.format(string))

    #%%     
    def start_eye_image_recording(self, image_name, path):
        ''' Starts eye image recording
        Example: start_eye_image_recording('test',"c:\\eyeimages\\" )
        '''
        
        self.send_command(' '.join(["ET_EVB 1", image_name, path]))

            
    #%%             
    def stop_eye_image_recording(self):
        ''' Stops eye image recording
        '''
        self.send_command("ET_EVE")
               
     
    #%%             
    def get_latest_sample(self):
        ''' Gets most recent gaze sample
        '''
        #pickle.dump(self.get_sample(), open( "sample.p", "wb" ) )
        
        return self.get_sample()
        
        
    #%% 
    def get_headbox_coordinates(self):
        ''' Get headbox coordinates
        '''
        return self.get_tracking_status()
        
    #%%              
    def increment_trial_number(self):
        ''' Increments trial number in iview X buffer.
        '''
        self.send_command("ET_INC")

    
    #%%  
    @WINFUNCTYPE(None, CSample)
    def sample_callback(sample):
        ''' Callback function for sample data
        '''
        # Append data to buffer
        global buf
        buf.append(sample)
    #%%
    def consume_buffer_data(self):
        ''' Get data from the online buffer. The returned samples are removed 
        from the buffer
        '''
        return buf.get_all()
        
    def peek_buffer_data(self):
        ''' Get data from the online buffer. The returned samples remain in 
        the buffer
        '''
        return buf.peek()
        
        
    def clear_buffer_data(self):
        ''' Clears buffer. 
        '''
        buf.clear()
    #%% 
    def start_buffer(self, sample_buffer_length=3):
        '''Start recording eye-movement data into buffer for online use
        
        Args:
            sample_buffer_length - size of buffer in samples
        '''
        
        # Initialize the ring buffer
        global buf
        buf = helpers.RingBuffer(maxlen=sample_buffer_length)
        print(buf)
        #print(buf, type(buf))
        self.__buffer_active = True
        
        # Set callback (starts reading samples)
        #self.set_sample_callback('sample_callback')
        res = iViewXAPI.iV_SetSampleCallback(self.sample_callback)
        
    #%% 
    def stop_buffer(self):
        '''Stops sample buffer''' 
        
        self.__buffer_active = False
        
        # Set callback to None
        res = iViewXAPI.iV_SetSampleCallback(None)     
            
    #%%             
    def record_eye_images(self,name = 'img', dur = 1, recorded_eye = 0):
        '''
        Records eye images (without overlays for dur s)
        recorded_eye = 0 actually means right eye (wrong in SDK)
        ''' 
        
        self.stop_recording() 
        self.set_tracking_parameter(recorded_eye,3,0)
        self.set_tracking_parameter(recorded_eye,4,0)
        self.set_tracking_parameter(recorded_eye,5,0)
        core.wait(0.1)
          
        self.start_eye_image_recording(name)
        core.wait(dur)
        self.stop_eye_image_recording()    
        
        self.set_tracking_parameter(recorded_eye,3,1)
        self.set_tracking_parameter(recorded_eye,4,1)
        self.set_tracking_parameter(recorded_eye,5,1)  
  
        
    #%%                         
    def enable_bilateral_filter(self):
        '''
        This API bilateral filter was implemented due to special human-computer
        interaction (HCI) application requirements. It smoothes gaze position data in EyeDataStruct::gazeX and
        EyeDataStruct::gazeY contained in SampleStruct, e.g. obtained by iV_GetSample. The gaze data filter
        can be disabled using iV_DisableGazeDataFilter      
        '''
        self.enable_gaze_data_filter()
        
    #%%             
    def disable_bilateral_filter(self):        
        '''
        Disables bilateral filter
        '''
        self.disable_gaze_data_filter()
                
        
    #%% 
    def delete_temp_idf_file(self):
        ''' Remove temp idf-files (otherwise the iview
            server complains that there are unsaved data)
        '''
        
        try:
            subprocess.Popen('del /F /S /Q /A r"C:\\ProgramData\\SMI\\iView X\\temp\\*.idf"') # RED-m
            subprocess.Popen('del /F /S /Q /A r"C:\\ProgramData\\SMI\\TempRemoteRecordings\\*.idf"') # RED NG


        except:
            print('Could not delete temp idf files')
            
    #%%     
    def de_init(self, close_et_server=False):
        ''' Close connection to the eye tracker and clean up
        Args:
            close_et_server - closes the eye tracker server application
        '''
        
        self.disable_processor_high_performance_mode()
        self.disconnect()
            
        if close_et_server:
            self.quit_server()
            
    #%%             
    def average_data(self, average = False):
        ''' Average data from both eyes.
        ''' 
        
        if average:
            self.configure_filter(filter_type=1, filter_action=1)
        else:
            self.configure_filter(filter_type=0, filter_action=1)
                                            
    




#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Load required packages 
from psychopy import core, event, misc, visual
import numpy as np
import SMITE_Dummy_raw
        
class Connect(object):
    """
    Creates a class that simplifies life for people wanting to use the SDK
    """
    def __init__(self):

        self.rawSMI = SMITE_Dummy_raw.Connect()
        
        # Mouse object
        self.mouse = event.Mouse()
        
        # clock
        self.clock = core.Clock()
     
    def init(self):
        ''' Connect to eye tracker
        and apply settings
        '''
        self.rawSMI.init()           
        self.geom = None
        self.system_info = None
        self.calibration_history = None
    #%%    
    def is_connected(self):
        self.rawSMI.is_connected()
    
    #%% Init calibration
    def calibrate(self, win):
        ''' Master function for setup and calibration
        '''
        
        # Window and instruction text for calibration        
        instruction_text = visual.TextStim(win,text='',wrapWidth = 1,
                                           height = 0.05, units='norm')  
        
        self.instruction_text = instruction_text
        
        self.win = win
        self.rawSMI.win = win
        
        self.instruction_text.text = 'Calibration Dummy mode'
        self.instruction_text.draw()
        self.win.flip()
        core.wait(2)
                
    #%%  
    def start_recording(self):
        self.rawSMI.start_recording()
        
    #%% 
    def start_buffer(self, sample_buffer_length=3):
        self.rawSMI.start_buffer(sample_buffer_length=sample_buffer_length)
        
    #%% 
    def send_message(self, msg):
        self.rawSMI.send_image_message(msg)
        
    #%%
    def get_latest_sample(self):
        sample  = self.rawSMI.get_latest_sample()
        return sample
        
    #%%
    def consume_buffer_data(self):
        data = self.rawSMI.consume_buffer_data()
        return data
    
    #%%
    def peek_buffer_data(self):
        data = self.rawSMI.peek_buffer_data()
        return data    
    
    #%% 
    def stop_buffer(self):
        self.rawSMI.start_buffer()    
        
    #%%  
    def stop_recording(self):
        self.rawSMI.stop_recording()
    #%% 
    def save_data(self, filename, description = "", 
                   user = None, overwrite=0):
        self.rawSMI.save_data(filename, description = "", 
                   user = None, overwrite=0)        
    #%%
    def de_init(self):
        self.rawSMI.de_init()
        
    #%%
    def set_begaze_trial_image(self, imname):
        self.rawSMI.set_begaze_trial_image(imname)
    #%%        
    def set_begaze_key_press(self, msg):
        self.rawSMI.set_begaze_key_press(msg)
    #%%
    def set_begaze_mouse_click(self, which, x, y):
        self.rawSMI.set_begaze_mouse_click(which, x, y)      
        
    #%%
    def start_eye_image_recording(self, image_name, path):
        self.rawSMI.start_eye_image_recording(image_name, path)
    #%%
    def stop_eye_image_recording(self, image_name, path):
        self.rawSMI.start_eye_image_recording(image_name, path)        
        
    #%% 
    def set_dummy_mode(self):
        print('Does nothing')



   
 


# -*- coding: utf-8 -*-
"""
Created on Sat Jan 09 19:50:57 2016

@author: marcus

Settings file for the REDm Professional eye tracker

"""

MY_MONITOR = 'default'              # needs to exist in PsychoPy monitor center

#%% CONNECTION AND TRACKER PARAMS

# Connection information. '127.0.0.1' indicates a one computer setup
port_listen=4444
port_send=5555
ip_send = '127.0.0.1'
ip_listen = '127.0.0.1'

track_mode = 'SMART_BINOCULAR'      # Tracking mode
sampling_freq = 60                  # Sampling rate of eye tracker

geom_profile = 'Default Profile'    # Geometric profile 

average_data = False                # Average gaze data across eyes
filtering = False                   # Use filter to smooth online data

#%% Files and filepaths
delete_temp_idf_files = False       # Delete temporary stored idf files
eye_image_size = (240, 300)         # No eye images with the eye tracker, just
                                    # a placeholder

#%% CALIBRATION PARAMETERS 
autoaccept = 1                      # autoaccept (2), semi autoaccept (1, accept first point) 
                                    # of accept with space bar (0)
                                    
n_cal_points = 5                    # number of calibration points supported: [0, 1, 2, 5, 9, 13] 

cal_speed = 0                       # pacing of calibration / validation targets [slow: 0, fast: 1]
                                    # 0: slow, 1:fast
select_best_calibration = True      # option to run a few calibration and 
                                    # select the best
                                    
reset_calibration_points = False    # Resets calibration points to default 
                                    # locations before calibration starts
                                    
record_data_during_calibration = True # Record data during calibration to idf-file

animate_calibration = False          # Show static points or animated targets
screen = 0                          # Display stimuli on a second screen attached 
                                    # to your computer (1). Single setup (0).
# Parameters to shift and scale the calibration grid
shift_cal_grid_x = 0
shift_cal_grid_y = 0
scale_cal_grid = 1












#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Load required packages 
from psychopy import core, event, visual
from iViewXAPI import*
from iViewXAPIReturnCodes import* 
import numpy as np
import datetime
from collections import deque
from scipy import misc
import calibration_graphics as graphics
import helpers
from ctypes import *
import SMITE_raw


global buf
      
class Connect(object):
    """ Create a class that simplifies life for people wanting to use the SDK
    """
    def __init__(self, in_arg):
        '''
        Args:
            in_arg: either
                (a) string with eye tracker name, e.g., 'HiSpeed'
                (b) or settings, where 
                    settings = SMITE.get_defaults(eye_tracker_name) 

        '''
              
        # A list to store validation results
        self.calibration_history = []
        
        self.clock = core.Clock()
        
        # Connect to a class to use 'raw' SMI functionality
        rawSMI = SMITE_raw.Connect(in_arg)
        self.rawSMI = rawSMI
        self.constants = rawSMI.constants
        self.eye_tracker_name = rawSMI.eye_tracker_name
        
    #%%
    def get_options(self):
        ''' Returns current/active settings
        '''
        return self.constants           
    #%%% 
    def set_options(self):
        ''' Change current/active settings
        '''        
        pass
         
    
    #%%  
    def init(self):
        ''' Connect to the SMI eye tracker and initialize it according to 
        the requested settings
        '''
        self.rawSMI.init()        
        self.geom = self.rawSMI.get_current_RED_geometry()
        self.system_info = self.rawSMI.get_system_info()
       
    #%%    
    def is_connected(self):
        ''' Report status of the connection to the eye tracker
        
        Returns:
            rawSMI.is_connected() (boolean)
        '''
        
        return self.rawSMI.is_connected()
    
    #%% Init calibration
    def calibrate(self, win):
        ''' Do participant setup and calibration, and validation
        
        Args: 
            win - a PsychoPy window
        '''
        
        self.screen_res = win.size
        self.screen_refresh_rate = win.getActualFrameRate()
        self.mouse = event.Mouse()
        
        # Scale or shift grid or calibration points?
        # Do only if shift or scaling required in the settings file
        if np.any([self.constants.shift_cal_grid_x, 
                   self.constants.shift_cal_grid_y]) or self.constants.scale_cal_grid != 1:
    
            # Reset calibration point to default values
            self.rawSMI.reset_calibration_points()
            
            # Get positions of current calibration points
            cal_pos = []
            for p in np.arange(self.constants.n_cal_points) + 1:
                cal_point_info = self.rawSMI.get_calibration_point(p)
                
                
                cal_pos.append([cal_point_info.positionX, 
                                cal_point_info.positionY])
    
            # Normalize point to -0.5->0.5 so they can be scaled
            cal_pos = np.array(cal_pos)
            cal_pos[:, 0] = cal_pos[:, 0] / self.screen_res[0] - 0.5
            cal_pos[:, 1] = cal_pos[:, 1] / self.screen_res[1] - 0.5
            
            # Scale 
            cal_pos[:, 0] = cal_pos[:, 0] * self.constants.scale_cal_grid
            cal_pos[:, 1] = cal_pos[:, 1] * self.constants.scale_cal_grid
            
            # Rescale to pixels
            cal_pos[:, 0] = (cal_pos[:, 0] + 0.5) * self.screen_res[0]
            cal_pos[:, 1] = (cal_pos[:, 1] + 0.5) * self.screen_res[1]
            
            # ... and shift
            cal_pos[:, 0] = cal_pos[:, 0] + self.constants.shift_cal_grid_x
            cal_pos[:, 1] = cal_pos[:, 1] + self.constants.shift_cal_grid_y            
            
            # Change to new positions
            for p in np.arange(self.constants.n_cal_points):
                self.rawSMI.change_calibration_point(p + 1, 
                                                     cal_pos[p, 0],
                                                     cal_pos[p, 1])

        # Make the window available for all calibration functions
        self.win = win        
        
        # Create click buttons used during calibration
        self._create_calibration_buttons(win)
        
        # Animated calibration?
        if self.constants.animate_calibration:
            
            # Define your calibration target
            target = helpers.MyDot(self.win, units='pix',
                                     outer_diameter = win.size[0] * 0.02, 
                                     inner_diameter = win.size[0] * 0.005)
            self.animator = AnimatedCalibrationDisplay(self.win, target, 'animate_point')
        
        # Main control loop
        action = 'setup'
        deviations = [] # List to store validation accuracies
        while True:
            
            if 'setup' in action:
                action = self._check_head_position()
            elif 'cal' in action:                                              
                success, deviations = self._run_calibration(deviations)
                if success:
                    action, selected_calibration = self._show_validation_screen(deviations)
                else:
                    action = 'setup'
            elif 'adv' in action:
                action = self._advanced_setup()
            elif 'done'in action:
                print('calibration completed')
                break
            elif 'quit' in action:
                self.win.close()
                core.quit()
                
            core.wait(0.1)
            
        # Save all calibrations (appending 1 means that the calibration was used)
        for i, devs in enumerate(deviations):
            if i == selected_calibration-1:
                self.calibration_history.append(devs + ['used'])
                
                # Write the calibration results to the idf file
                self.start_recording()
                self.send_message("Calibration results in degrees (LX, LY, RX, RY): {}".format([d for d in devs]))
                self.stop_recording()
                
            else:
                self.calibration_history.append(devs + ['not used'])  
              
    #%%  
    def start_recording(self, clear_buffer=False):
        ''' Start recording eye-movement data to idf file
        
        Args:
            clear_buffer - clear IDF buffer (removes all data)
        '''
        
        if clear_buffer:
            self.rawSMI.clear_recording_buffer()
            
        self.rawSMI.start_recording()
        
    #%% 
    def start_buffer(self, sample_buffer_length=3):
        '''Start recording eye-movement data into buffer for online use
        
        Args:
            sample_buffer_length - size of buffer in samples
        '''
        
        self.rawSMI.start_buffer(sample_buffer_length=sample_buffer_length)
        
    #%% 
    def send_message(self, msg):
        ''' Insert message into idf file
        
        Args: 
            msg - message string to put into buffer
        '''
        
        self.rawSMI.send_image_message(msg)
        
    #%%
    def get_latest_sample(self):
        ''' Get most recent data sample
        
        Returns:
            sample 
            
            Ex: sample.leftEye.gazeY
            See SMI manual for full description of what sample contains
        '''
        sample  = self.rawSMI.get_latest_sample()
        return sample
        
    #%%
    def consume_buffer_data(self):
        ''' Get data from the online buffer. The returned samples are removed 
        from the buffer
        
        Returns:
            data - list of samples (see get_latest_sample)
        '''
        data = self.rawSMI.consume_buffer_data()
        return data
    
    #%%
    def peek_buffer_data(self):
        ''' Get data from the online buffer. The returned samples remain in 
        the buffer
        
        Returns:
            data - list of samples (see get_latest_sample)        
        '''
        data = self.rawSMI.peek_buffer_data()
        return data    
    
    #%% 
    def stop_buffer(self, clear_buffer=False):
        ''' Stop recording data into buffer
        '''
        self.rawSMI.stop_buffer()    
        if clear_buffer:
            self.rawSMI.clear_buffer_data()
        
    #%%  
    def stop_recording(self):
        ''' Stop recording data into idf file 
        '''
        self.rawSMI.stop_recording()
        
        
    #%% 
    def save_data(self, filename, description = "", 
                   user = None, append_version=True):
        ''' Save idf file to specified location
        The data recording needs to be stopped using iV_StopRecording
        before the data buffer can be saved to given location. 
        The filename can include the path. If the connected eye tracking device 
        is an HED, scene video buffer is written, too. iV_SaveData will not return
        until the data has been saved.
        
        Args:
            filename - full path including the filename of the data file being created
            description - Optional experiment description tag stored in the idf file. This tag is available in BeGaze and in the text export from an idf file.
            user - Optional name of test person. This tag is available in BeGaze and in the text export
                    from an idf file.
            append_version - append version number to file if exists.            
                If True, and there is already a file with a certain name 'name.idf', this file will not 
                be overwritten, but save as another file with name 'name_1.idf'.            
        
        '''
        self.rawSMI.save_data(filename, description = description, 
                   user = user, append_version=append_version)        
    #%%
    def de_init(self):
        ''' Close connection to the eye tracker and clean up
        '''
        self.rawSMI.de_init()
        
    #%%
    def set_begaze_trial_image(self, imname):
        ''' Put specially prepared message in idf file to notify BeGaze what 
        stimulus image/video belongs to a trial
        
        Args:
            imname - filename of stimulus that is shown on this trial. 
            Must have one of the following extentions: .png, .jpg, .jpeg, .bmp, 
            or .avi
        '''
        
        self.rawSMI.set_begaze_trial_image(imname)
    #%%        
    def set_begaze_key_press(self, msg):
        ''' Put specially prepared message in idf file that shows up as 
        keypress in BeGaze
        
        Args: 
            msg - string that will show up on BeGaze's event timeline. 
            Can be name of a key, but also other arbitrary string.
        '''
        self.rawSMI.set_begaze_key_press(msg)
    #%%
    def set_begaze_mouse_click(self, which, x, y):
        ''' Put specially prepared message in idf file that shows up as mouse 
        click in BeGaze.
        
        Args:
            which: string indicating which mouse button, left or right
            x: horizontal coordinate of mouse click
            y: vertical coordinate of mouse click
        '''
        self.rawSMI.set_begaze_mouse_click(which, x, y)
        
    #%%
    def start_eye_image_recording(self, image_name, path):
        ''' Start recording eye images to file. Not supported on RED250mobile, 
        REDn Scientific, and REDn Professional.
        
        Args:
            image_name - filename where recorded eye images will be saved
            path - path where the eye images are stored
            
        Example: start_eye_image_recording('test',"c:\\eyeimages\\" )
        '''
        self.rawSMI.start_eye_image_recording(image_name, path)
    #%%
    def stop_eye_image_recording(self, image_name, path):
        ''' Stop recording eye images to file 
        '''
        self.rawSMI.start_eye_image_recording(image_name, path)        
        
    #%% 
    def set_dummy_mode(self):
        ''' Enable dummy mode, which allows running the program without an 
        eye tracker connected
        '''
        import SMITE_Dummy, SMITE_Dummy_raw
        self.__class__ = SMITE_Dummy.Connect
        self.__class__.__init__(self)    
        self.rawSMI = SMITE_Dummy_raw.Connect()

    #%% 
    def _create_calibration_buttons(self, win):
        '''Creates click buttons that are used during calibration
        
        Args: 
            win - PsychoPy window
        '''
        
        # Find out ratio aspect of (16:10) or 
        screen_res = win.size
        ratio = screen_res[0] / float(screen_res[1])
        
        # Shown during setup to check gaze in corners
        self.POS_CAL_CHECK_DOTS = [[-0.45 * ratio, -0.45], [0.45 * ratio, -0.45], 
                              [-0.45 * ratio, 0.45], [0.45 * ratio, 0.45]]

        # Text object to draw text (on buttons)
        #text_size = 0.04
        instruction_text = visual.TextStim(win,text='',wrapWidth = 1,height = graphics.TEXT_SIZE, units='norm')  
        self.instruction_text = instruction_text
        
        # Setup stimuli for drawing calibration / validation targets
        self.cal_dot = helpers.MyDot2(win, units='pix',
                                     outer_diameter=graphics.TARGET_SIZE, 
                                     inner_diameter=graphics.TARGET_SIZE_INNER)
        
        # Click buttons
        self.calibrate_button = visual.Rect(win, width= graphics.WIDTH_CAL_BUTTON, 
                                            height=graphics.HEIGHT_CAL_BUTTON,  
                                        units='norm', fillColor=graphics.COLOR_CAL_BUTTON,
                                        pos=graphics.POS_CAL_BUTTON)                
        self.calibrate_button_text = visual.TextStim(win, text=graphics.CAL_BUTTON_TEXT, 
                                                     height=graphics.TEXT_SIZE, units='norm',
                                                     pos=graphics.POS_CAL_BUTTON)
                                                     
        self.recalibrate_button = visual.Rect(win, width= graphics.WIDTH_RECAL_BUTTON, 
                                            height=graphics.HEIGHT_RECAL_BUTTON,  
                                        units='norm', fillColor=graphics.COLOR_RECAL_BUTTON,
                                        pos=graphics.POS_RECAL_BUTTON) 
        self.recalibrate_button_text = visual.TextStim(win, text=graphics.RECAL_BUTTON_TEXT, 
                                                     height=graphics.TEXT_SIZE, units='norm',
                                                     pos=graphics.POS_RECAL_BUTTON)                                                     
                                        
        self.setup_button = visual.Rect(win, width= graphics.WIDTH_SETUP_BUTTON, 
                                        height=graphics.HEIGHT_SETUP_BUTTON,  
                                        units='norm', fillColor=graphics.COLOR_SETUP_BUTTON,
                                        pos=graphics.POS_SETUP_BUTTON)
        self.setup_button_text = visual.TextStim(win, text=graphics.SETUP_BUTTON_TEXT, 
                                                 height=graphics.TEXT_SIZE, units='norm',
                                                 pos=graphics.POS_SETUP_BUTTON)             

        self.accept_button = visual.Rect(win, width= graphics.WIDTH_ACCEPT_BUTTON, 
                                        height=graphics.HEIGHT_ACCEPT_BUTTON,  
                                        units='norm', fillColor=graphics.COLOR_ACCEPT_BUTTON,
                                        pos=graphics.POS_ACCEPT_BUTTON)
        self.accept_button_text = visual.TextStim(win, text=graphics.ACCEPT_BUTTON_TEXT, 
                                                  height=graphics.TEXT_SIZE, units='norm',
                                                  pos=graphics.POS_ACCEPT_BUTTON)             
                                                                        
        self.back_button = visual.Rect(win, width= graphics.WIDTH_BACK_BUTTON, 
                                        height=graphics.HEIGHT_BACK_BUTTON,  
                                        units='norm', fillColor=graphics.COLOR_BACK_BUTTON,
                                        pos=graphics.POS_BACK_BUTTON)    
        self.back_button_text = visual.TextStim(win, text=graphics.BACK_BUTTON_TEXT, 
                                                height=graphics.TEXT_SIZE, units='norm',
                                                pos=graphics.POS_BACK_BUTTON)             
                                                                      
        self.gaze_button = visual.Rect(win, width= graphics.WIDTH_GAZE_BUTTON, 
                                        height=graphics.HEIGHT_GAZE_BUTTON,  
                                        units='norm', fillColor=graphics.COLOR_GAZE_BUTTON,
                                        pos=graphics.POS_GAZE_BUTTON)   
        self.gaze_button_text = visual.TextStim(win, text=graphics.GAZE_BUTTON_TEXT, 
                                                height=graphics.TEXT_SIZE, units='norm',
                                                pos=graphics.POS_GAZE_BUTTON)             
                                        
        # Dots for the setup screen
        self.setup_dot = helpers.MyDot(win, units='height',
                                     outer_diameter=graphics.SETUP_DOT_OUTER_DIAMETER, 
                                     inner_diameter=graphics.SETUP_DOT_INNER_DIAMETER)        
        
        # Setup control circles for head position
        self.static_circ = visual.Circle(win, radius = graphics.HEAD_POS_CIRCLE_FIXED_RADIUS, 
                                         lineColor = graphics.HEAD_POS_CIRCLE_FIXED_COLOR,
                                         lineWidth=4, units='height')
        self.moving_circ = visual.Circle(win, radius = graphics.HEAD_POS_CIRCLE_MOVING_RADIUS, 
                                         lineColor = graphics.HEAD_POS_CIRCLE_MOVING_COLOR,
                                         lineWidth=4, units='height')        
                                         
        # Dot for showing et data
        self.et_sample_l = visual.Circle(win, radius = graphics.ET_SAMPLE_RADIUS, 
                                         fillColor = 'blue', units='pix') 
        self.et_sample_r = visual.Circle(win, radius = graphics.ET_SAMPLE_RADIUS, 
                                         fillColor = 'red', units='pix')                                         
      
        # Show images (eye image, validation resutls)
        self.eye_image_stim = visual.GratingStim(win, units='pix', size=graphics.EYE_IMAGE_SIZE,
                                                    tex=np.zeros(graphics.EYE_IMAGE_SIZE))
        
        # Accuracy image 
        self.accuracy_image = visual.ImageStim(win, image=None,units='norm', size=(1.5,1.5),
                                          pos=(0, 0))
        
        # Tracking monitor image
        self.tracking_monitor = visual.ImageStim(win, image=None,units='norm', size=(0.5,0.5),
                                          pos=(0, -0.5), ori = 180)
        

                        
    #%%      
    def _check_head_position(self):
        ''' Check to make sure that the head is in the center of the track box
        Two circles are shown; one static (representing the center of the headbox)
        and one variable representing the head position. The head is in the center 
        of the trackbox the circles overlap.
        
        Four dot are shown in the corners. Ask the participants to fixate them
        to make sure there are not problems in the corners of the screen
        '''
        xyz_pos_eye_l = (0, 0, 0)
        xyz_pos_eye_r = (0, 0, 0)
        
        while True:
            
            # Draw four dots in the corners
            for i in self.POS_CAL_CHECK_DOTS:
                self.setup_dot.setPos(i)
                self.setup_dot.draw()

            # Draw buttons, one to calibrate, and one to 
            # go to the advanced setup screen
            self.calibrate_button.draw()
            self.calibrate_button_text.draw()   
            
            k = event.getKeys()
                
            # Plot distance information only for remotes (not HiSpeed)
            if 'RED' in self.eye_tracker_name:
                self.setup_button.draw()
                self.setup_button_text.draw()            
                
                if graphics.SETUP_BUTTON in k or self.mouse.isPressedIn(self.setup_button, buttons=[0]):
                    # Go to advanced setup screen
                    action = 'adv'
                    break     
                                 
                # Get information about where the head is in the head box
                # and visualize it
                headbox_data = self.rawSMI.get_headbox_coordinates()

                # Get position of eye in track box (average of both eyes)
                xyz_pos_eye_l = (headbox_data.leftEye.relativePositionX,
                                 headbox_data.leftEye.relativePositionY,
                                 headbox_data.leftEye.relativePositionZ)
                xyz_pos_eye_r = (headbox_data.rightEye.relativePositionX,
                                 headbox_data.rightEye.relativePositionY,
                                 headbox_data.rightEye.relativePositionZ)
              
                # Get the average head position
                avg_pos = []
                for i in np.arange(len(xyz_pos_eye_r)):
                    avg_pos.append((xyz_pos_eye_l[i] + xyz_pos_eye_r[i]) / 2.0)   
                    
                # Draw moving circle based on the data (0.5, 0.5, 0.5)  means the 
                # eye is in the center of the box
                self.moving_circ.pos = avg_pos[0], avg_pos[1]
                self.moving_circ.radius = avg_pos[2] * 0.30 + graphics.HEAD_POS_CIRCLE_FIXED_RADIUS
    
                # Restrict min and max size of circles
                if self.moving_circ.radius < 0.01:
                    self.moving_circ.radius = 0.01
                    
                if self.moving_circ.radius > 0.3:
                    self.moving_circ.radius = 0.3
    
                # Draw circles
                self.moving_circ.draw()
                self.static_circ.draw()
                            
                # Draw instruction
                self.instruction_text.pos = (0, 0.8)
                self.instruction_text.text = 'Position yourself such that the two circles overlap.'
                self.instruction_text.draw()
                
                # Get and draw distance information
                sample_data = self.rawSMI.get_latest_sample()
                l_pos = sample_data.leftEye.eyePositionZ
                r_pos = sample_data.rightEye.eyePositionZ
                
                try:
                    self.instruction_text.pos = (0, 0.7)
                    self.instruction_text.text = ' '.join(['Distance:', str(int((l_pos + r_pos)/2.0/10)), 'cm'])
                    self.instruction_text.draw() 
                except:
                    pass
            else:
                # 'Hidden' button in the HiSpeed mode
                if graphics.SETUP_BUTTON in k:
                    action = 'adv'
                    break 
                    
            # check whether someone left clicked any of the buttons or pressed 'space'
            if graphics.CAL_BUTTON in k or self.mouse.isPressedIn(self.calibrate_button, buttons=[0]):
                action = 'cal'
                break
                
            if 'escape' in k:
                action = 'quit'
                break
                    
            # Option to skip calibration (no button for this?)
            if 'return' in k:
                action = 'done'
                break               
            
            # Update the screen
            self.win.flip()
         
        return action
        
    #%%        
    def _run_calibration(self, deviations, show_instructions=False,
                   select_best_calibration=False, optional=False):
        """ Run the calibration
        
        Args: 
            deviations - list containing results from previous calibrations
            show_instruction - show calibration instructions 
            select_best_calibration - option to store and select from all performed
                                        calibrations
            optional - is the calibration optional or required (allows you to 
                       skip the calibration if set to True).
        """
                
        self.win.flip()
        self.instruction_text.pos = (0, 0)
        self.instruction_text.color = (-1, -1, -1)
        
        print('calibration started')
        
        # Optional calibration
        if optional:
            self.instruction_text.setText('Press q to calibrate or any other key to continue')
            self.instruction_text.draw()
            self.win.flip()
            k = event.waitKeys()
            if not 'q' in k[0]:   
                return False, deviations
                
        # Show calibration instructions
        if show_instructions:
            self.instruction_text.setText(
                'A number of dots will be presented on the screen. Carefully look in the center of each dot for as long as it is shown.\n '
                '(press space to begin).')
            self.instruction_text.draw()
            self.win.flip()
            event.waitKeys()
                  
        print('calibration_start')
        
        validation_successful = False
        while not validation_successful:
            # Calibrate using PsychoPy (dots are drawn by PsychoPy)
            calibration_state = self._calibrate_custom(mode = 'cal')
            
            # if the calibration is successful, start a validation
            if calibration_state == 1:
                calibration_state = self._calibrate_custom(mode = 'val')
                if calibration_state == 1:
                    validation_successful = True
               
            # if the calibration failed (someone pressed escape, return unsuccessful calibration)
            if calibration_state == 2:
                break
                
        # Get accuracy values (only if validation success)
        if validation_successful:
            accuracy = self.rawSMI.get_accuracy()
            print('calibration_end')
            
            # Save calibration Display accuracy values
            now = datetime.datetime.now() # Get current time
            deviations.append([str(accuracy[0])[:4],
                               str(accuracy[1])[:4],
                               str(accuracy[2])[:4],
                               str(accuracy[3])[:4],
                               now.isoformat()])
            
        return validation_successful, deviations
    #%%     
    def _advanced_setup(self):
        ''' Shows eye image and tracking monitor
        Good if you are having problems in the circle setup, 
        and want to see what is wrong.
        
        '''
        print('advanced mode')
        
        # Keep track of whether overlays are displayed
        toggle_state_pupil = False
        toggle_state_cr = False
   
        action = 'setup'
        
        # Create toggle buttons for pupil and CR crosshairs
        toggle_buttons = []
        n_toggle_Buttons = 2 # pupil, CR
        
        button_pos = ((0.5, 0), (0.5, -0.1))
        button_text = ['pupil (p)', 'CR (c)']
        for i in range(n_toggle_Buttons):
            toggle_buttons.append(visual.Rect(self.win, width= 0.15, 
                                                height= 0.06, 
                                                units='norm',
                                                fillColor=graphics.blue,
                                                pos=button_pos[i]))
        
        dist = 0
        done = False

        while not done:
            
            if 'HiSpeed' not in self.eye_tracker_name:
                
                # Get eye tracker distance 
                sample_data = self.get_latest_sample()
                dist = (sample_data.leftEye.eyePositionZ + 
                        sample_data.rightEye.eyePositionZ) / 2.0
                
                # Draw instruction
                self.instruction_text.pos = (0, 0.7)
                self.instruction_text.text = ' '.join(['Distance:', str(dist / 10.0)[:2], 'cm'])
                self.instruction_text.draw()
                                       
                # Draw buttons
                self.back_button.draw()
                self.back_button_text.draw()
            
            # Draw calibration button for all systems
            self.calibrate_button.draw()
            self.calibrate_button_text.draw()
            
            # Draw four dots in the corners
            for i in self.POS_CAL_CHECK_DOTS:
                self.setup_dot.setPos(i)
                self.setup_dot.draw()               
            
            # Check for keypress or mouse click to toggle visibility of pupil and CR corsshairs
            # of move on with setup
            k = event.getKeys()
            
            # The option to toggle pupil and CR buttons is only available in the REDm
            if self.eye_tracker_name == 'REDm':
                
                # Draw toggle buttons and text
                for i in range(n_toggle_Buttons):
                    toggle_buttons[i].draw()
                    self.instruction_text.text = button_text[i]
                    self.instruction_text.pos = button_pos[i]
                    self.instruction_text.draw()
                
                # Check if mouse is clicked on a toggle button
                button_clicked = -1
                for i, button in enumerate(toggle_buttons):
                    if self.mouse.isPressedIn(button):
                        button_clicked = i
                
                # Check whether keys or mouse are pressed/clicked
                if 'p' in k or button_clicked == 0:
                    if self.clock.getTime() > 1:
                        if toggle_state_pupil:
                            activate = 0
                            toggle_buttons[0].fillColor = graphics.blue
                        else:
                            activate = 1
                            toggle_buttons[0].fillColor = graphics.blue_active
                        
                        toggle_state_pupil = not toggle_state_pupil                    
                        res = self.rawSMI.set_tracking_parameter(eye_type=2,
                                                           parameter_type=4,
                                                           activate=activate)
                        self.clock.reset()
    
                elif 'c' in k or button_clicked == 1:  
                    if self.clock.getTime() > 1:
                        if toggle_state_cr:
                            activate = 0
                            toggle_buttons[1].fillColor = graphics.blue
                        else:
                            activate = 1
                            toggle_buttons[1].fillColor = graphics.blue_active
                        
                        toggle_state_cr = not toggle_state_cr                    
                        res = self.rawSMI.set_tracking_parameter(eye_type=2, 
                                                           parameter_type=5, 
                                                           activate=activate)
                                                           
                        self.clock.reset()
            
            # Draw eye image
            eye_image, res = self.rawSMI.get_eye_image()
            if res == 1:
                self.eye_image_stim.tex = eye_image

            self.eye_image_stim.draw()
              
            # Show tracking monitor (not available for HiSpeed)
            if 'HiSpeed' not in self.eye_tracker_name: 
                tracking_monitor_image, res = self.rawSMI.get_tracking_monitor()
                if res == 1:
                    self.tracking_monitor.image = tracking_monitor_image
                self.tracking_monitor.draw()
                
                # Check keypress and mouse clicks
                if graphics.BACK_BUTTON in k or self.mouse.isPressedIn(self.back_button, buttons=[0]):
                    action = 'setup'
                    break
                
            # Check keypress and mouse clicks
            if graphics.CAL_BUTTON in k or self.mouse.isPressedIn(self.calibrate_button, buttons=[0]):
                action = 'cal'
                break

                
            self.win.flip()

        return action
    #%%     
    def _show_validation_screen(self, deviations):
        ''' Shows validation image after a validation has been completed.
        The validation screen includes both visual and numerical feedback 
        about the calibration.
        '''    
        
        # Center position of presented calibration values
        x_pos_res = 0.4
        y_pos_res = 0
        
        # get (and save) validation screen image
        nCalibrations = len(deviations)
        fname = str(nCalibrations)+'.jpg'
        im = self.rawSMI.get_accuracy_image(fname) 
        
        # Save calibration 
        self.rawSMI.save_calibration(str(nCalibrations))
        core.wait(0.2)
        
        # Add image as texture
        self.accuracy_image.image = fname
        
        # information about data quality header
        header = ['Accuracy', 'LX', 'LY', 'RX', 'RY']
        x_pos= np.linspace(-0.20, 0.20, num = 5)     
        
        # Prepare header
        header_text = []    
        for j, x in enumerate(x_pos):
            header_text.append(visual.TextStim(self.win,text=header[j],
                                                wrapWidth = 1,
                                                height = graphics.TEXT_SIZE, 
                                                units='norm',
                                                pos = (x, 0.1),
                                                color = (1, 1, 1)))        
        
        # Prepare rects for buttons, button text, and accuracy values
        select_accuracy_rect = []
        select_rect_text = []
        accuracy_values = []

        y_pos = y_pos_res
        #print(self.deviations)
        for i in range(nCalibrations):
            #print(self.deviations[i])
            select_accuracy_rect.append(visual.Rect(self.win, width= 0.15, 
                                                height= 0.05, 
                                                units='norm',
                                                pos = (x_pos_res, y_pos)))
                                                
            select_rect_text.append(visual.TextStim(self.win,
                                                    text='Select',
                                                    wrapWidth = 1,
                                                    height = graphics.TEXT_SIZE, 
                                                    units='norm',
                                                    pos = (x_pos_res, y_pos)))  
                    
            # Then prepare the accuracy values for each calibration preceded 
            # by Cal x (the calibration number)
            accuracy_values_j = []
            for j, x in enumerate(x_pos):       
                #print(deviations)
                if j > 0:
                    accuracy_values_j.append(visual.TextStim(self.win,
                                                        text='{0:.2f}'.format(float(deviations[i][j - 1])),
                                                        wrapWidth = 1,
                                                        height = graphics.TEXT_SIZE, 
                                                        units='norm',
                                                        pos = (x, y_pos),
                                                        color = (1, 1, 1)))                
                else:
                    accuracy_values_j.append(visual.TextStim(self.win,
                                                        text='Cal' + str(i+1) + ':',
                                                        wrapWidth = 1,
                                                        height = graphics.TEXT_SIZE, 
                                                        units='norm',
                                                        pos = (x, y_pos),
                                                        color = (1, 1, 1))) 
            accuracy_values.append(accuracy_values_j)                 
            y_pos -= 0.06
        
        # Wait for user input
        selected_calibration = nCalibrations # keep track of which calibration is selected
        selection_done = False
        display_gaze = False
        gaze_button_pressed = False
        while not selection_done:
                        
            # Draw validation results image
            self.accuracy_image.draw()
                                   
            # Draw buttons (re-calibrate, accept and move on, show gaze)
            self.recalibrate_button.draw()
            self.recalibrate_button_text.draw()
            
            self.accept_button.draw()
            self.accept_button_text.draw()
            
            self.gaze_button.draw()
            self.gaze_button_text.draw()            
                        
            # Now draw accuracy values
            if self.constants.select_best_calibration:
                    
                # Print a list of deviations from all calibrations and choose one to continue
                # ToDo (mark selected text)
                ins = ''
               
                # Draw header
                [h.draw() for h in header_text]
                    
                # Draw accuracy values and define rectangles to select a calibration
                y_pos = 0
                
                for i in range(nCalibrations):
                    
                   # Highlight selected calibrations
                    if i == selected_calibration - 1: # Calibration selected
                        select_accuracy_rect[i].fillColor = graphics.blue_active
                        if nCalibrations > 1:
                            select_accuracy_rect[i].draw() 
                            select_rect_text[i].draw()     
                    else:
    #                    select_rect_text[i].color = graphics.blue
                        select_accuracy_rect[i].fillColor = graphics.blue
                        select_accuracy_rect[i].draw() 
                        select_rect_text[i].draw()  
                    
                    # Then draw the accuracy values for each calibration preceded 
                    # by Cal x (the calibration number)                    
                    for j, x in enumerate(x_pos): 
                        accuracy_values[i][j].draw()          
                        
                        
                    y_pos -= 0.06
                        
            else:        
                ins = deviations[-1]
                        
            # Check if mouse is clicked to select a calibration
            for i, button in enumerate(select_accuracy_rect):
                if self.mouse.isPressedIn(button):
                    self.rawSMI.load_calibration(str(i + 1))  # Load the selected calibration
                    fname = str(i + 1) + '.jpg'
                    self.accuracy_image.image = fname
                    selected_calibration = int(i + 1)
                    break
                    
            
            # Check if key or button is pressed
            k = event.getKeys()
            if graphics.RECAL_BUTTON in k or self.mouse.isPressedIn(self.recalibrate_button):
                action = 'setup'
                selection_done = True
            elif graphics.ACCEPT_BUTTON in k or self.mouse.isPressedIn(self.accept_button):
                action = 'done'
                selection_done = True                
            elif k:
                if k[0].isdigit():
                    if any([s for s in range(nCalibrations+1) if s == int(k[0])]):
                        self.rawSMI.load_calibration(k[0])  # Load the selected calibration
                        fname = str(k[0]) + '.jpg'
                        self.accuracy_image.image = fname
                        selected_calibration = int(k[0])       
                        
            elif 'escape' in k:
                action = 'quit'
                break
                
            # Toggle display gaze
            if graphics.GAZE_BUTTON in k or (self.mouse.isPressedIn(self.gaze_button, buttons=[0]) and not gaze_button_pressed):
                display_gaze = not display_gaze
                gaze_button_pressed = True
                
            # Check whether mouse button released
            if not np.any(self.mouse.getPressed()):
                gaze_button_pressed = False
              
            # Display gaze along with four dots in the corners
            if display_gaze:
                for i in self.POS_CAL_CHECK_DOTS:
                    self.setup_dot.setPos(i)
                    self.setup_dot.draw()
                self._draw_gaze()
        
            self.win.flip() 

        # Clear screen and return
        self.instruction_text.color = (1, 1, 1)
        self.win.flip()
        
        return action, selected_calibration
    

    #%%             
    def _calibrate_custom(self, mode = 'cal'):
        '''
        Draw own calibration dots in PsychoPy instead of using visualizations by SMI
        Used for both valication and calibration. 
        
        Args:
            mode - calibrate ('cal') or validate ('val')
        '''
        
        # If in validation mode, and semiautomatic mode is used, skip manual accept of first point
        temp_autoaccept = self.constants.autoaccept
        if 'val' in  mode and self.constants.autoaccept == 1:
            temp_autoaccept = 2
            self.rawSMI.setup_calibration_parameters(autoaccept=temp_autoaccept,
                                          cal_method=self.constants.n_cal_points, 
                                          cal_speed=self.constants.cal_speed,
                                          screen = self.constants.screen)
            
        if 'cal' in mode:
            self.rawSMI.calibrate_iview()
        else:
            self.rawSMI.validate_iview()
        
        # Record data during calibration and validation?
        if self.constants.record_data_during_calibration:
            self.start_recording()
            now = datetime.datetime.now()
            self.send_message(mode + 'started: ' + now.isoformat())
        
        calibration_done = False 
        calibration_state = 1 # 0 - calibration should be restarted
                              # 1 - calibration/validation succeded
                              # 2 - calibration/validation should be interrupted. Return to setup screen
        
        p_old = -1
        p = -1
        tick = 0
        pos_old = (0, 0)
        x_old, y_old = 0, 0
        animation_state = 'static' # or move
        while not calibration_done:
                     
            # Update currentCalibrationPoint struct
            res, currentCalibrationPoint = self.rawSMI.get_current_calibration_point()
            
            # Draw and show calibration point 
            if res == 1:
                
                # Get information about current calibration point and draw it
                p = currentCalibrationPoint.number
                x = currentCalibrationPoint.positionX
                y = currentCalibrationPoint.positionY
                
                # if the calibration point has changed
                if p != p_old:
                    
                    # Write information about dot position to idf file
                    if self.constants.record_data_during_calibration:              
                        now = datetime.datetime.now()
                        self.send_message('_'.join([mode, str(p), str(x), str(y), now.isoformat()]))
                    p_old = p
                    pos_old = (x_old, y_old)
                    tick = 0
                    animation_state = 'move'                         
               
                # Convert to psychopy coordinates
                xy = helpers.smi2psychopy(np.array([[x, y]]), self.win.monitor, units='pix')
                
                # Animate calibration dots or show static dots?
                if self.constants.animate_calibration:
                    if animation_state == 'move':                    
                        move_completed = self.animator.move_point(pos_old, (xy[:, 0], xy[:, 1]), tick)
                        if move_completed:
                            animation_state = 'static' # or move
                            tick = 0
                    else:
                        self.animator.animate_target(p, (xy[:, 0], xy[:, 1]), tick)
                else:
                    self.cal_dot.setPos(xy)
                    self.cal_dot.draw()
                
                self.win.flip()
                    
                x_old = xy[:, 0]
                y_old = xy[:, 1]
                
            # if no new data available, quit calibration.
            elif res == 2:
                calibration_done = True
            else: # If other messages occur, ignore them (sometimes complaints of connection loss)
                pass
                
            # Accept calibration point
            k = event.getKeys()      
            if temp_autoaccept == 2: # Automatic accept
                pass
            elif temp_autoaccept == 0: # Manual accept
                if 'space' in k and self.clock.getTime() > 0.5:
                    self.rawSMI.accept_calibration_point()
                    self.clock.reset()
            else: # if first point, wait for keypress, otherwise continue automatically
                if p == 1: 
                    if 'space' in k:
                        self.rawSMI.accept_calibration_point()
             
            # Abort calibration if esc is pressed (and go back to setup screen)
            if 'escape' in k:
                self.rawSMI.abort_calibration()
                calibration_state = 2
                break
                
            # 'r' aborts and restarts calibration 
            if 'r' in k:
                tick = 0
                self.rawSMI.abort_calibration()                          
                calibration_state = 0
                break
             
            tick += 1
        
        # Stop recording (if recording)
        if self.constants.record_data_during_calibration:
            now = datetime.datetime.now()
            self.send_message(mode + 'stopped: ' + now.isoformat())                    
            self.stop_recording()
            
        self.win.flip()
        event.clearEvents()
        
        # Change back to default point acceptence mode
        if 'val' in  mode and self.constants.autoaccept == 1:
            self.rawSMI.setup_calibration_parameters(autoaccept=self.constants.autoaccept,
                                          cal_method=self.constants.n_cal_points, 
                                          cal_speed=self.constants.cal_speed,
                                          screen = self.constants.screen)            
        
        return calibration_state
        

    #%%                           
    def _show_gaze(self):
        ''' Display gaze data until a keypress
        '''
        while True:
            self._draw_gaze()
            k = event.getKeys()
            if k:
                break
                
            #core.wait(1.0 / self.screen_refresh_rate)
            self.win.flip()
    #%%             
    def _draw_gaze(self):
        '''  Draws gaze data as circles for both eyes
        '''
            
        # Get latest gaze data and draw them
        sample_data = self.get_latest_sample()
                        
        xl = sample_data.leftEye.gazeX
        yl = sample_data.leftEye.gazeY 
        xr = sample_data.rightEye.gazeX
        yr = sample_data.rightEye.gazeY
        
        pos = helpers.smi2psychopy(np.array([[xl, yl]]), self.win.monitor)
        self.et_sample_l.pos = (pos[:, 0][0], pos[:, 1][0])
        self.et_sample_l.draw()
        
        pos = helpers.smi2psychopy(np.array([[xr, yr]]), self.win.monitor)
        self.et_sample_r.pos = (pos[:, 0][0], pos[:, 1][0])
        self.et_sample_r.draw()            

#%%   
class AnimatedCalibrationDisplay(object):
    """ A class for drawing animated targets"""
    def __init__(self, win, target, function_name):
        ''' The function 'function_name' does the actual drawing
        '''
        self.win = win
        self.function_name = function_name
        self.target = target # psychopy.visual object (should be in 'pix' units)
        self.screen_refresh_rate = float(win.getActualFrameRate())
        
    def animate_target(self, point_number, position, tick):
        ''' Calls the target drawing function 'func'
        '''

        eval(''.join(['self.',self.function_name,'(',str(point_number),',', 
                                        '(',str(position[0][0]),',',
                                            str(position[1][0]), ')' ,',', str(tick),')']))
        
            
    def animate_point(self, point_number, position, tick):
        ''' Animates calibration point with a certain point_number and position
        tick increases by 1 for each call
        Args:
            position - (x, y)
        '''
        
        target_size = np.abs(1 - np.sin(3 * tick / self.screen_refresh_rate + 3*np.pi/2)) + 0.2
        self.target.setSize(target_size)
        self.target.setPos(position)
        self.target.draw()
        
    def move_point(self, old_position, new_position, tick):
        ''' Animates movement between two positions
        '''
        move_completed = False
        
        # The target should have a fixed size when moving
        self.target.setSize(2)
        
        # How many ticks should the movement be (one screen unit in one second)?
        n_steps = self.screen_refresh_rate / 2
        step_pos_x = np.linspace(old_position[0], new_position[0], n_steps)
        step_pos_y = np.linspace(old_position[1], new_position[1], n_steps)
        
        if tick >= len(step_pos_x):
            move_completed = True
        else:       
            self.target.setPos((step_pos_x[tick], step_pos_y[tick]))
            
        self.target.draw()
        
        return move_completed

        
        
        
        
    



# -*- coding: utf-8 -*-
"""
Created on Fri Nov 10 21:31:13 2017

@author: marcus
"""

from psychopy import visual
import numpy as np
from psychopy.tools import monitorunittools
from collections import deque
import copy

#%% Toggle button
class MyDot2:
    '''
    Generates the best fixation target according to Thaler et al. (2013)
    '''
    def __init__(self, win, outer_diameter=0.5, inner_diameter=0.1,
                 outer_color = 'white', inner_color = 'black',units = 'pix'):
        '''
        Class to generate a stimulus dot with 
        units are derived from the window
        '''

        # Set propertis of dot 
        outer_dot = visual.Circle(win,fillColor = outer_color, radius = outer_diameter/2.0,
                                  units = units)
        inner_dot = visual.Circle(win,fillColor = outer_color, radius = inner_diameter/2.0,
                                  units = units)
        line_vertical = visual.Rect(win, width=inner_diameter, height=outer_diameter, 
                                    fillColor=inner_color, units = units)
        line_horizontal = visual.Rect(win, width=outer_diameter, height=inner_diameter, 
                                    fillColor=inner_color, units = units)        
              
    
        self.outer_dot = outer_dot
        self.inner_dot = inner_dot
        self.line_vertical = line_vertical
        self.line_horizontal = line_horizontal
    

    def setSize(self, size):
        self.outer_dot.size = size
        self.line_vertical.size = size
        self.line_horizontal.size = size
        
    def setPos(self,pos):
        '''
        sets position of dot
        pos = [x,y]
        '''
        self.outer_dot.pos = pos
        self.inner_dot.pos = pos
        self.line_vertical.pos = pos
        self.line_horizontal.pos = pos        
        
    def get_pos(self):
        '''
        get position of dot
        '''
        pos = self.outer_dot.pos 
        
        return pos

    
    def draw(self):
        '''
        draws the dot
        '''
        self.outer_dot.draw()
        self.line_vertical.draw()
        self.line_horizontal.draw()           
        self.inner_dot.draw()
     
        
    def invert_color(self):
        '''
        inverts the colors of the dot
        '''
        temp = self.outer_dot.fillColor 
        self.outer_dot.fillColor = self.inner_dot.fillColor
        self.inner_dot.fillColor = temp
        
    def set_color(self, color):
        self.outer_dot.lineColor = 'blue'
        self.outer_dot.fillColor = 'blue'
        self.inner_dot.fillColor = 'blue'
        self.inner_dot.lineColor = 'blue'
        self.line_vertical.lineColor = 'red'
        self.line_horizontal.fillColor = 'red'
        self.line_vertical.fillColor = 'red'
        self.line_horizontal.lineColor = 'red'        
        

#%%
class MyDot:
    '''
    Generates a fancy looking dot 
    '''
    def __init__(self, win,outer_diameter=0.5, inner_diameter=0.1,
                 outer_color = 'white', inner_color = 'black',
                 units = 'pix'):
        '''
        Class to generate a stimulus dot with 
        units are derived from the window
        '''

        # Set propertis of dot 
        outer_dot = visual.Circle(win,fillColor = outer_color, radius = outer_diameter/2.0, units=units)
        inner_dot = visual.Circle(win,fillColor = inner_color, radius = inner_diameter/2.0, units=units)
              
    
        self.outer_dot = outer_dot
        self.inner_dot = inner_dot
    

    def setSize(self, size):
        self.outer_dot.size = size
        #self.inner_dot.radius = size
        
    def setPos(self,pos):
        '''
        sets position of dot
        pos = [x,y]
        '''
        self.outer_dot.pos = pos
        self.inner_dot.pos = pos

    
    def draw(self):
        '''
        draws the dot
        '''
        self.outer_dot.draw()
        self.inner_dot.draw()
        
    def invert_color(self):
        '''
        inverts the colors of the dot
        '''
        temp = self.outer_dot.fillColor 
        self.outer_dot.fillColor = self.inner_dot.fillColor
        self.inner_dot.fillColor = temp
        
def pixels2degrees(pixels,viewing_dist = 0.70,screen_res = [1280,1024],
                   screen_size = [0.38,0.30],dim = 'h',center_coordinate_system = False):
    '''
    Converts from pixels to degrees
    Pixels can be a single value or a 1D array
    to ensure that conversion is correct in vertical and horizontal dimensions
    '''

    # Conversion in horizontal or vertical dimension?
    if dim == 'h':
        res =  screen_res[0]
        size = screen_size[0]
    else:
        res = screen_res[1]
        size = screen_size[1]
    
    # SMIs coordinate-system starts in the upper left corner and psychopy's in 
    # the center of the screen
    if center_coordinate_system:
        pixels = pixels - res/2        
        
    meter_per_pixel = size/res
    meters = pixels*meter_per_pixel
    
    alpha = 180/np.pi*(2*np.arctan(meters/viewing_dist/2))
    
    return alpha        
    
def smi2psychopy(pos, mon, units = 'pix'):
    ''' Converts from smi output (pixels, origo in upper left corner, positive y downward) to 
    psychopy coordinate system  (origo in center, positive y upware)
    Args:   pos: N x 2 array 
            mon: psychopy monitor            
    '''
    screen_res = mon.getSizePix()
    
    # Center coordinate system to center of screen (now in psychopy 'pix')
    pos[:, 0] = pos[:, 0] - screen_res[0]/2.0
    pos[:, 1] = (pos[:, 1] - screen_res[1]/2.0) *-1
    
    # convert to [-1, 1]
    if 'norm' in units:
        pos[:, 0] = pos[:, 0] / screen_res[0]/2.0
        pos[:, 0] = pos[:, 1] / screen_res[1]/2.0
    elif 'deg' in units:
        pos[:, 0] = monitorunittools.pix2deg(pos[:, 0], mon, correctFlat=False)
        pos[:, 1] = monitorunittools.pix2deg(pos[:, 1], mon, correctFlat=False)
    elif 'pix':
        pass
    else:
        raise IOError ('Unvalid unit')

    return pos

def psychopy2smi(pos, mon, units='norm'):
    ''' Converts from psychopy norm coordinate system to smi coords (pix).
    Args:   pos: N x 2 array 
            mon: psychopy monitor            
    '''
    
    screen_res = mon.getSizePix()    
    
    # Change units to pixels and 
    if 'norm' in units:
        pos[:, 0] = pos[:, 0] * screen_res[0]/2.0 
        pos[:, 1] = pos[:, 1] * screen_res[1]/2.0 
    if 'deg' in units:
        pos[:, 0] = monitorunittools.deg2pix(pos[:, 0], mon, correctFlat=False)
        pos[:, 1] = monitorunittools.deg2pix(pos[:, 1], mon, correctFlat=False)        
        
    # Move origo to upper left (y downward)
    pos[:, 0] = pos[:, 0] + screen_res[0]/2.0 
    pos[:, 1] = -1 * pos[:, 1] + screen_res[1]/2.0 
    
    return pos
           

#%%         
class RingBuffer(object):
    """ A simple ring buffer based on the deque class
    Used as online buffer of gaze samples"""
    def __init__(self, maxlen=200):        
        # Create que with maxlen 
        self.maxlen = maxlen
        self._b = deque(maxlen=maxlen)  

    def clear(self):
        """ Clears buffer """
        return(self._b.clear())
        
    def get_all(self):
        """ Returns all samples from buffer and empties the buffer"""
        lenb = len(self._b)
        return([self._b.popleft() for i in range(lenb)])
        
    def peek(self):
        """ Returns all samples from buffer without emptying the buffer
        First remove an element, then add it again
        """
        b_temp = copy.copy(self._b)
        c = []
        if len(b_temp) > 0:
            for i in xrange(len(b_temp)):
                c.append(b_temp.pop())

        return(c)
        
    def append(self, L):
        self._b.append(L)         
        """"Append buffer with the most recent sample (list L)"""



# -*- coding: utf-8 -*-
"""
Created on Mon Feb 26 14:16:59 2018

@author: Marcus
"""

from psychopy import core

def get_defaults(eye_tracker_name):
    ''' Returns default setting
    '''
    print(eye_tracker_name)
    if eye_tracker_name == 'REDm':
        import REDm as settings
    elif eye_tracker_name == 'HiSpeed':
        import HiSpeed as settings
    elif eye_tracker_name == 'RED':
        import RED as settings
    elif eye_tracker_name == 'REDn_Professional':
        import REDn_Professional as settings
    elif eye_tracker_name == 'REDn_Scientific':
        import REDn_Scientific as settings        
    elif eye_tracker_name == 'RED250mobile':
        import RED250mobile as settings             
    else:
        print('Eye tracker not defined')
        core.quit()   
        
    settings.eye_tracker_name = eye_tracker_name
    
    return settings
    
    
class Connect(object):   
    def __init__(self, in_arg='dummy'):
        '''  in_arg can be either string with the eye tracker name
        or 'settings' generated by calling (and optionally modifying)
        the output from get_defaults()
        '''
        
        if isinstance(in_arg, str):
            if 'dummy' in in_arg:
                import SMITE_Dummy
                self.__class__ = SMITE_Dummy.Connect
                self.__class__.__init__(self)  
            else:            
                import SMITE_ET
                self.__class__ = SMITE_ET.Connect
                self.__class__.__init__(self, in_arg)
        else:
            import SMITE_ET
            self.__class__ = SMITE_ET.Connect
            self.__class__.__init__(self, in_arg)
            
            
        

